﻿var RepositoryObject = function () {
	"use strict";
	
	var args = arguments;
	
	this.Data = {};
	this.InstantiationLevel = [];
	this.TemplateSettings = [];

	if (typeof args[0] != "undefined") {
		if (args[0] == null) {
			return;
		}
		if (typeof args[0] == "object") {
			this.merge(args[0]);
		}
		else if (typeof args[0] == "string") {
			var oid = args[0].split("/");
			this.Data.ObjId = oid[0];
		    	if (oid[1]) {
				this.Data.RevisionId = oid[1];
			}
			
			if ($.pageCache("get", oid[0], true)) {
				this.Data = $.pageCache("get", oid[0]);
			}
			
			this.cache("get");
		}
	}
	
	if (this.Data.Template && !this.Type) {
		$.extend(true, this, $TemplateSettings[this.Data.Template]);
	}
	
	if (!this.TemplateDefinition && $.pageCache("get", "def" + this.Data.ObjId, true)) {
		this.TemplateDefinition = $.pageCache("get", "def" + this.Data.ObjId);
	}
}

RepositoryObject.prototype.auditattributes = function() { this.auditAttributes.apply(this, arguments); }
RepositoryObject.prototype.auditAttributes = function() {
	var attributes = [
		"AuditCD", 
		"AuditCN", 
		"AuditMD", 
		"AuditMN",
		"Language", 
		"Revision", 
		"RevisionId", 
		"ObjectId", 
		"Name", 
		"Template",
		"Type"
	];
	
	return attributes;
}

RepositoryObject.prototype.governanceattributes = function() { this.governanceAttributes.apply(this, arguments); }
RepositoryObject.prototype.governanceAttributes = function() {
	var attributes = [
		"GovernanceCycleCount", 
		"GovernanceCycleOwner", 
		"GovernanceDateLimitOnState", 
		"GovernanceLimitDate",
		"GovernanceOnStatusSince", 
		"GovernanceRecyclingCount", 
		"GovernanceSignatureDone", 
		"GovernanceSignatureMissing", 
		"GovernanceSLA", 
		"GovernanceStatus",
		"GovernanceState",
		"GovernanceWarningDate",
		"GovernanceWorkFlow",
		"GweCurrentState"
	];
	
	return attributes;
}

RepositoryObject.prototype.genericattributes = function() { this.genericAttributes.apply(this, arguments); }
RepositoryObject.prototype.genericAttributes = function() {
	var attributes = [
		"Contains", 
		"ContainedIn", 
		"Links", 
		"LinksTo",
		"LinkedBy"
	];
	
	return attributes;
}

RepositoryObject.prototype.graphicalattributes = function() { this.graphicalAttributes.apply(this, arguments); }
RepositoryObject.prototype.graphicalAttributes = function() {
	var attributes = [
		"Frames", 
		"FramedBy",
		"From",
		"ReceivesFrom",
		"Receives",
		"Sends",
		"SendsTo",
		"To"
	];
	
	return attributes;
}

RepositoryObject.prototype.info = function() { this.Info.apply(this, arguments); }
RepositoryObject.prototype.Info = function() {
	var $obj = this;
	var attr = arguments[0] ? arguments[0] : null;
	var calb = arguments[1] ? arguments[1] : $.noop;
	var mthd = arguments[2] ? arguments[2] : "ServiceHandlers.GetAttributeValue";
	var cach = true;
	
	if (typeof arguments[3] != "undefined") {
		cach = arguments[3];
	}

	if (!attr) {
		calb.call($obj, {});
	}
		
	if (typeof attr == "string") {
		attr = attr.replace(/ /g,"").split(/,|;/g);
	}
	
	if (cach) {
		$obj.cache("get");
		$.each(attr, function(i,a) {
			if (a in $obj.Data) {
				attr.shift();
			}
		})		
		if (!attr.length) {
			calb.call($obj, $obj.Data);
			return;
		}
	}
	
	var status = $obj.InstantiationLevel.match("status").length;
	if (!status) {
		$.each($obj.governanceAttributes, function(idx, val){
			if ($.inArray(val, attr)>-1) {
				status = true;
				attr.splice(idx,1);
			}
		});
		if (status) {
			$obj.status(function(){
				$obj.info.call($obj, attr, callb, mthd, cach);
			});
			return;
		}
	}
	
	if (mthd === "ServiceHandlers.GetAttributeValue") {
		if ($.inArray("RemoteFile", attr)>-1
		|| $.inArray("FileAttachment", attr)>-1
		|| $.inArray("FileName", attr)>-1) {
			attr.push("FileName", "RemoteFile", "FileAttachment", "MediaType", "FileOrigin");
		}
	}
		
	attr = attr.unique();
	
	var attrlist = attr.unique().join(",");
	var attrstr = attrlist.replace(/,/g,"-");
	
	if ($obj.cache("info", attrstr)) {
		$obj.delegate(function(){
			calb.call($obj, $obj.Data);
		}, "info", attrstr);
		return;
	}

	$QisSession.Init[$obj.Data.ObjId + "-info-" + attrstr] = 1;
	
	var wsurl = initQisCsharpMethod.call($obj, mthd); 
	var parms = window.JSON.stringify({ 
		objectAttributes:[	
			{Name:"sys_ObjectId", Format:"PlainText", Value: $obj.Data.ObjId},
			{Name:"attribute", Format:"PlainText", Value: attrlist}
		]
	});
	
	$.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		url: wsurl,
		data: parms
	})
	.done(function (data) {
		if (!isValidJsonData.call($obj, data, mthd)) {
			delete $QisSession.Init[$obj.Data.ObjId + "-info-" + attrstr];
			return;				
		}

		$.browser.log("loading attribute info[" + $obj.Data.ObjId + "(" + attrlist + ")] ...");

		var $data = data.length ? $.parseJSON(data) : {};
		$obj.cache("get")
		
		$.each(attr, function(idx, attribute) {
			delete $obj.Data[attribute];
			var $value = $data[attribute];
			
			if (!$value) {
				$obj.Data[attribute] = null;
				return true;
			}
			
			$obj.Data[attribute] = {};
			
			switch ($value.Format) {
				case "null": break;
				
				case 2:
					$obj.Data[attribute].Format = "PlainText";
					$obj.Data[attribute].Value = $value.Text;
					break;
					
				case 3:
					$obj.Data[attribute].Format = "Xhtml";
					$obj.Data[attribute].Value = $value.Text;
					break;
					
				case 4: 
					$obj.Data[attribute].Format = "SingleLink";
					$obj.Data[attribute].Value = [{
						Name: $value.HyperLink.Text,
						ObjId: $value.HyperLink.ObjectId.Id.Uid,
						RepId: $value.HyperLink.RepositoryId.Value
					}];
					break;
										
				case 6:
					$obj.Data[attribute].Format = "Date";
					$obj.Data[attribute].Value = $obj.Date($value.UtcValue);
					break;
					
				case 9:
					$obj.Data[attribute].Format = "SingleFile";
					$obj.Data[attribute].Value = "<dns:link src=\"" 
						+ $value.HyperLink.Uri +"\" type=\"FileLink\" xmlns:dns=\"http://qualiware.com/qis/diagram\">" 
						+ $value.HyperLink.Text +"</dns:link>";
					break;
					
				default:
					if (typeof $value === "string") {
						$obj.Data[attribute] = $value;
					}
					else if ($value.length) {
						$obj.Data[attribute].Format = "MultiLink";
						$obj.Data[attribute].Value = [];
						$.each($value, function(idx, $v) {							
							$obj.Data[attribute].Value.push({
								Name: $v.Text,
								ObjId: $v.ObjectId.Id.Value,
								RepId: $v.RepositoryId ? $v.RepositoryId.Value : $QisSession.RepId
							});
							if ($v.CustomAttributes) {
								$obj.Data[attribute].Value.LinkValues = $v.CustomAttributes.LinkValue 
									? $v.CustomAttributes.LinkValue.split("\t")
									: [];
							}
						});
					}
					break;
			}
		});
		
		$obj.cache(true);
		delete $QisSession.Init[$obj.Data.ObjId + "-info-" + attrstr];
		
		calb.call($obj, $obj.Data);
	})
	.fail(function (jqxhr, textStatus, error) {
		delete $QisSession.Init[$obj.Data.ObjId + "-info-" + attrstr];
		
		if (!jqxhr.readyState && !jqxhr.status) {
			$obj.cache(false);
			handleRequestError(jqxhr, textStatus, error, wsurl, $obj);
		}
		else if (!jqxhr.responseText) {
			$obj.cache(false);
			calb.call($obj, false);
		}
		else if (jqxhr.status == 400){
			notifyRequestError(jqxhr, textStatus, error, wsurl, parms, $obj);
		}
	});
}

RepositoryObject.prototype.audit = function () { this.Audit.apply(this, arguments); }
RepositoryObject.prototype.Audit = function () { 
	var $obj = this;
	var calb = arguments[0] ? arguments[0] : $.noop;
	var notf = arguments[1] ? arguments[1] : null;
	var cach = arguments[2] ? arguments[2] : false;
	var mthd = "ServiceHandlers.GetAuditInfo";
	
	if ($obj.cache("audit")) {
		$obj.delegate(function(){
			calb.call($obj, $obj.Data);
		}, "audit");
		return;
	}
	
	if (cach) {
		var $tmp = $obj.cache("get");
		if ($tmp) {
			if ($obj.InstantiationLevel.match("audit").length) {
				$obj.merge($tmp);
				calb.call($obj, $obj.Data); 
				return;
			}
		}
	}

	$QisSession.Init[$obj.Data.ObjId + "-audit"] = 1;
	
	var wsurl = initQisCsharpMethod.call($obj, mthd);
	var parms = window.JSON.stringify({ 
		objectAttributes:[{Name:"sys_ObjectId", Format:"PlainText", Value: $obj.Data.ObjId}] 
	});
	
	$.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		url: wsurl,
		data: parms
	})
	.done(function (data) {
		if (!isValidJsonData.call($obj, data, mthd)) {
			data = "";
		}
		
		$.browser.log("loading object audit[" + $obj.Data.ObjId + "] ...");

		var $data = data.length ? $.parseJSON(data) : {};		
		$obj.cache("get");
		
		if ((!$obj.Data.AuditMD && $data.AuditMD) 
		|| ($obj.Data.AuditMD < $data.AuditMD) 
		|| !$obj.InstantiationLevel.match("audit").length) {
			$obj.cache(false);
			$obj.merge($data);
			$obj.InstantiationLevel.clear().push("audit");
			$obj.cache(true);
			if (notf) {
				$obj.Expired = true;
				$.jnotify({
					message: typeof notf == "function" ? notf.call($obj) : notf, 
					persistent: true, 
					reload: true,
					callback: function(){
						$.pageCache("clear");
						$obj.init(function(){
							$(".jgrowl-container .reload").remove();
							$.initPageContent(null, true);
						}, true); 
					}
				});
			}
		}
		
		delete $QisSession.Init[$obj.Data.ObjId + "-audit"];
		calb.call($obj, $data);
	})
	.fail(function (jqxhr, textStatus, error) {
		if (!jqxhr.readyState && !jqxhr.status) {
			handleRequestError(jqxhr, textStatus, error, wsurl, null, $obj);
			$obj.cache(false);
			
		}
		else if (!jqxhr.responseText) {
			$obj.cache(false);
		}
		else if (jqxhr.status == 400){
			notifyRequestError(jqxhr, textStatus, error, wsurl, parms);
		}
		$obj.InstantiationLevel.clear().push("audit");

		delete $QisSession.Init[$obj.Data.ObjId + "-audit"];
		calb.call($obj, false);
	})
}

RepositoryObject.prototype.init = function () { this.Init.apply(this, arguments); }
RepositoryObject.prototype.Init = function () {
	var $obj = this;
    var calb = arguments[0] ? arguments[0] : $.noop;
    var poll = arguments[1] ? arguments[1] : false;
    var notf = arguments[2] ? arguments[2] : false;
    var forc = arguments[3] ? arguments[3] : false;
	
	var wsurl = null;
	
	if (!$obj.Data.ObjId) { 
		calb.call($obj);
		return;
	}
		
	if (!forc) {
		if ($obj.cache("get")) {
			if ($obj.InstantiationLevel.match("data").length) {
				calb.call($obj, $obj.Data); 
				return;
			}
		}
	}
	
	if ($obj.cache("init")) {
		$obj.delegate(function(){
			calb.call($obj, $obj.Data);
		});
		return;
	}

	wsurl = initQisWebService({
		objid: $obj.Data.ObjId,
		confid: $obj.Data.ConfId,
		repid: $obj.Data.RepId
	});
	
	$QisSession.Init[$obj.Data.ObjId + "-init"] = 1;
	
    $.getJSON(wsurl)
        .done(function (data) {
		if (!isValidJsonData.call($obj, data, "init")) {
			data = "";
		}
		
		$.browser.log("loading object data[" + $obj.Data.ObjId + "] ...");
				
		var $data = data.length ? $.parseJSON(data) : {};		
		$obj.merge($data);
		$obj.InstantiationLevel.clear().push("audit","data");
		$obj.Expired = false;
		$obj.cache(true);
			
		delete $QisSession.Init[$obj.Data.ObjId + "-init"];
			
		if (poll) {
			var ptime = parseInt(poll) || $QepSettings.PollInterval;
			var interval = $.async(function () {
				if ($obj.Expired === true) {
					$.async(false, interval);
					return;
				}
				$.browser.info("Poll request [" + $obj.Data.ObjId + "] ...");
				$obj.audit(null, function() {
					return $.LL("{Name} has been modified {AuditMD} by {AuditMN}", $obj, true) + "<br/><br/><i>" + $.LL("Close the notification to reload content") + "</i>"
				});
			}, ptime, true);
			
			$QisSession.Init[$obj.Data.ObjId + "-poll"] = interval;
		}
		else if (notf) {
			$.jnotify({
				message: (typeof notf=="function" ? notf.call($obj) : notf)
			});
		}

		calb.call($obj, $data);
        })
	.fail(function (jqxhr, textStatus, error) {
		handleRequestError(jqxhr, textStatus, error, wsurl, null, $obj);
		$obj.InstantiationLevel.clear().push("audit","data");
		delete $QisSession.Init[$obj.Data.ObjId + "-init"];
		calb.call($obj, false);
	});
}

RepositoryObject.prototype.status = function () { this.Status.apply(this, arguments); }
RepositoryObject.prototype.Status = function () {
    var $obj = this;
    var calb = arguments[0] ? arguments[0] : $.noop;
    var forc = arguments[1] ? arguments[1] : false;
    var mthd = "GWE_ServiceHandlers.GetGovernanceStatus";
  
	if ($obj.cache("status")) {
		$obj.delegate(function(){
			calb.call($obj, $obj.Data);
		}, "status");
		return;
	}
	
	if (!forc) {
		if ($obj.cache("get")) {
			if ($obj.InstantiationLevel.match("status").length) {
				calb.call($obj, $obj.Data); 
				return;
			}
		}
	}
	
	$QisSession.Init[$obj.Data.ObjId + "-status"] = 1;
	
	var wsurl = initQisCsharpMethod.call($obj, mthd);
	var parms = window.JSON.stringify({
		objectAttributes:[{
			Name:"sys_ObjectId",Format:"PlainText",Value:$obj.Data.ObjId}
		]
	});
	
	$.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		url: wsurl,
		data: parms
	})
	.done(function (data) {
		if (!isValidJsonData.call($obj, data, mthd)) {
			data = "";
		}
		
		var $data = data.length ? $.parseJSON(data) : {};
		$obj.merge($data);
		$obj.cache(true);
		$obj.InstantiationLevel.insert("status");
		$obj.InstantiationLevel.sort();
		
		$obj.Data.GovernanceState = $obj.Data.GovernanceStatus;
		
		delete $QisSession.Init[$obj.Data.ObjId + "-status"];
		calb.call($obj, $data);
	})
	.fail(function (jqxhr, textStatus, error) {
		handleRequestError(jqxhr, textStatus, error, wsurl, parms, $obj);
		obj.InstantiationLevel.insert("status");
		$obj.InstantiationLevel.sort();
		delete $QisSession.Init[$obj.Data.ObjId + "-status"];
		calb.call($obj, false);
	});
}

RepositoryObject.prototype.merge = function () { this.Merge.apply(this, arguments); }
RepositoryObject.prototype.Merge = function () {
	if (!arguments[0]) {
		return;
	}

	var $tmp = arguments[0];
	
	if (typeof arguments[0] == "object") {
		var $obj = this;
		var $tmp = arguments[0];
		
		if ($tmp.LatestVisit) {
			$tmp.LatestVisit = (new Date($tmp.LatestVisit)).toLocaleString();
		}
		
		if( $tmp.nodeType == 1 || $tmp.jquery) {
			var $data = $tmp.jquery ? $tmp.data() : $($tmp).data();
			$tmp = {
				ObjId: $data.objid,
				RevisionId: $data.revisionid||$data.revid,
				Template: $data.template,
				Name: $data.name,
				Language: $data.language,
				ConfId: $data.confid,
				RepId: $data.repid
			}
			
			if (!this.Data.ObjId) {
				this.Data.ObjId = $tmp.ObjId;
				if ($.pageCache("get", $tmp.ObjId, true)) {
					this.Data = $.pageCache("get", $tmp.ObjId);
				}
				this.cache("get");
			}
		}
		
		if (arguments[1]) {
			var attrlist = ((typeof arguments[1] == "string") ? arguments[1].split(",") : arguments[1]);
			$.each(attrlist, function (idx, value) {
				$obj.Data[value=="RevId"?"RevisionId":value] = $tmp[value];
			})
		}
		else {
			if ($tmp.Data) {
				$.extend(true, $obj, $tmp);
			}
			else {
				$.extend(true, $obj.Data, $tmp);
			}
		}
		
		if (String($obj.Data.ObjId).indexOf("/") > -1) {
			var str = String($obj.Data.ObjId).split("/");
			$obj.Data.ObjId = str[0];
			$obj.Data.RevisionId = str[1];
		}
		if ($obj.Data.RevisionId != $obj.Data.RevId) {
			$obj.Data.RevisionId = $obj.Data.RevId || $obj.Data.RevisionId;
		}
		if ($obj.Data.Language !=  $obj.Data.LangCode) {
			$obj.Data.Language = $obj.Data.LangCode || $obj.Data.Language;
		}
		if ($obj.Data.ConfId != $obj.Data.Configuration) {
			$obj.Data.ConfId = $obj.Data.Configuration || $obj.Data.ConfId;
		}
		if ($obj.Data.RepId != $obj.Data.Repository) {
			$obj.Data.RepId = $obj.Data.Repository || $obj.Data.RepId;
		}
		
		delete $obj.Data.RevId;
		delete $obj.Data.LangCode;
		delete $obj.Data.Configuration;
		delete $obj.Data.Repository;
	}
	
	if (!this.Type) {
		$.extend(true, this, $TemplateSettings[this.Data.Template]);
	}

	if (!this.TemplateDefinition && $.pageCache("get", "def" + this.Data.ObjId, true)) {
		this.TemplateDefinition = $.pageCache("get", "def" + this.Data.ObjId);
	}
}

RepositoryObject.prototype.delegate = function () { return this.Delegate.apply(this, arguments);}
RepositoryObject.prototype.Delegate = function () {
	if (!arguments.length) {
		return;
	}
	
	var $obj = this;
	var callback = arguments[0] ? arguments[0] : $.noop;
	var option = arguments[1] ? arguments[1] : "init";
	var params = arguments[2] ? arguments[2] : "";

	if ( !$obj.cache(option, params) ){
		switch (option) {
			case "init": 
				$obj.InstantiationLevel.insert("audit", "data");
				$obj.InstantiationLevel.sort();
				break;
				
			case "audit": 
				$obj.InstantiationLevel.insert("audit");
				$obj.InstantiationLevel.sort();
				break;
				
			case "status": 
				$obj.InstantiationLevel.insert("status");
				$obj.InstantiationLevel.sort();
				break;
				
			default: break;
		}
		$obj.cache("get");
		return callback.call($obj)
	}
	
	$.async(function(){
		$obj.delegate(callback, option, params);
	}, 100);
}

RepositoryObject.prototype.cache = function () { return this.Cache.apply(this, arguments); }
RepositoryObject.prototype.Cache = function () {
	var $obj = this;
	var isCached = $.browserCache("get", $obj.Data.ObjId) || $.pageCache("get", $obj.Data.ObjId);

	if (typeof isCached == "object") {
		isCached = window.JSON.stringify(isCached);
	}
	
	if (arguments.length) {
		var option = arguments[0];
		switch (option) {
			case "get": 
				if (isCached) {
					$obj.merge($.parseJSON(isCached));
					return $obj;
				}
				else {
					return null;
				}
			
			case "audit":
			 	return $QisSession.Init[$obj.Data.ObjId + "-audit"];
			
			case "init":
			 	return $QisSession.Init[$obj.Data.ObjId + "-init"];
			 
			case "status":
			 	return $QisSession.Init[$obj.Data.ObjId + "-status"];
				
			case "info":
				var attr = arguments[1] ? arguments[1] : ""
				return $QisSession.Init[$obj.Data.ObjId + "-info-" + attr];
			 
			 default:
				if (option) {
					var exp = arguments[1] ? arguments[1] : $QepSettings.CacheTime;
					$.browserCache("set", $obj.Data.ObjId, window.JSON.stringify($obj, function(key, value) {
						if (typeof value === 'function') {
							return value.toString();
						} 
						else {
							return value;
						}
					}), exp);
					$obj.Expired = false;
				 }
				 else {
					$.browserCache("del", $obj.Data.ObjId);
					$obj.InstantiationLevel.clear();
					$obj.Expired = true;
				}
				return;
		}
	}
	
	return (isCached != null);
}

RepositoryObject.prototype.attr = function () { return this.Attr.apply(this, arguments); }
RepositoryObject.prototype.Attr = function () {
    if (!arguments.length) {
        return;
    }

    var $obj = this;

    var attrName = arguments[0];
    var attrFilter = arguments[1] || [];
    var attrStyle = arguments[2] || "";
    var $val = arguments[3] || "";
    var attrFormat = arguments[4] || "";
    var length = arguments[5] || false;
    var showLinkValue = arguments[6] || false;
    var $opts = arguments[7] || {};
	
    if (!attrName) {
        return $();
    }
	
  if (!$val) {
		$val = $("<div/>");
    }

    if (typeof attrName == "string") {
		if (attrName.match("{")) {
			$val.append($.LL(attrName, $obj))
				.find("a").each(function() {
					$(this).setTarget();
				});
			return $val;
		}
		
        attrName = attrName
			.replace(/CurrInst/gi,"")
			.replace(/-&lt;/gi,"-<")
			.replace(/-&gt;/gi,"->")
			.split(",");
		
		attrName = $.map(attrName, function(attribute){
			return ((attribute.indexOf("->") === 0 
			&& attribute.indexOf("-<") < 0)
				? attribute.replace("->", "")
				: attribute);
			});
    }

	if (typeof attrFilter === "string") {
		if (attrFilter!=="") {
			attrFilter = attrFilter!=="undefined" ?
				$.trim(attrFilter) :
				"";
		}
		if (attrFilter) {
			attrFilter = attrFilter.split(",");
		}
    }
	
	if (attrStyle!=="") {
		attrStyle = $.trim(attrStyle);
	}
	
	if (attrFormat!=="") {
		attrFormat = $.trim(attrFormat);
	}

    $.each(attrName, function (index, attr) {
		if (attr.match("(CurrGraph)")) {
			attr = attr.replace("(CurrGraph)","");
			$opts.diagram = $$ ? $$.Data.ObjId : null;
		}
		
		if (attr.indexOf("HTMLDrawing") === 0) {
			$obj.audit(function(valid){
				if (valid) {
					loadGraphicalData($val, $obj, true, null, true, null, true);
				}
			});
			return true;
		}
		else if (attr.indexOf("->") > -1 || attr.indexOf("-<") > -1 
		|| $obj.genericAttributes().match(attr).length
		|| $obj.graphicalAttributes().match(attr).length) {
			var idx1 = attr.indexOf("->");
			var idx2 = attr.indexOf("-<");
			
			if (idx1 < 0) {
				idx1 = attr.length;
			}
			if (idx2 < 0) {
				idx2 = attr.length;
			}
			
			var back_rel = (idx1 > idx2) ? true : false;
			var attr_arr = attr.split(/->|-</g).remove("");
			var attribute = attr_arr.shift();
			
			if (idx1 === 0 || idx2 === 0) {
				attr = attr.substring(2 + attribute.length);
			}
			else {
				attr = attr.substring(attribute.length);
			}
			
			attrFormat = "list"

			var queryCallback = function() {
				if (length) {
					$val.html(0);
				}
				$.each(this, function(idx, $obj1){
					var isRepositoryObject = ($obj1 instanceof RepositoryObject);
					if (attr && isRepositoryObject) {
						var cls = attr.replace(/-</gi, "-").replace(/->/gi, "-")
							.substring(1);
							
						return $obj1.attr(attr, attrFilter, attrStyle, $val, attrFormat, length, showLinkValue);
					}
					
					if (length) {
						$val.html(idx+1);
					}
					else if (isRepositoryObject){
						$val.tableOp($obj1.HyperLink(), attrFormat);
					}
				});
				
				$val.ajaxLoaderText(false);
				
				return;
			}
			
			$val.ajaxLoaderText();
			
			if (back_rel) {
				$obj.GenericRelation({
					relation: "LinkedBy",
					attribute: attribute,
					templateFilter: attrFilter,
					callback: function(){
						queryCallback.call(this);
					}
				});
			}
			else if ($obj.genericAttributes().match(attribute).length) {
				$obj.GenericRelation({
					relation: attribute,
					templateFilter: attrFilter,
					callback: function(){
						queryCallback.call(this);
					}
				});
			}
			else if ($obj.graphicalAttributes().match(attribute).length) {
				$obj.GraphRelation({
					relation: attribute,
					templateFilter: attrFilter,
					diagram: $opts.diagram,
					callback: function(){
						queryCallback.call(this);
					}
				});
			}
			else if (attr.length) {
				$obj.GenericRelation({
					relation: "LinksTo",
					attribute: attribute,
					templateFilter: attrFilter,
					callback: function(){
						queryCallback.call(this);
					}
				});
			}
			else {
				var $list = $obj.attr(attribute, "", "RepositoryObject");
				queryCallback.call($list);
			}
						
			return;
		}
		
		var property = $obj.Data[attr];
			
        if (typeof property == "undefined") {
			if ($obj.auditAttributes().match(attr).length) {
				if (!$obj.InstantiationLevel.match("audit").length) {
					$val.ajaxLoaderText();
					$obj.audit(function(valid) {
						if (valid) {
							$val.ajaxLoaderText(null, false);
							$obj.attr.call($obj, attrName, attrFilter, attrStyle, $val, attrFormat);
						}
					})
				}
			}
			else if ($obj.governanceAttributes().match(attr).length) {
				if (!$obj.InstantiationLevel.match("status").length) {
					$val.ajaxLoaderText();
					$obj.status(function() {
						$val.ajaxLoaderText(null, false);
						$obj.attr.call($obj, attrName, attrFilter, attrStyle, $val, attrFormat);
					})
				}
			}
			else if (!$obj.InstantiationLevel.match("data").length) {
				$val.ajaxLoaderText();
				$obj.init(function(){
					$val.ajaxLoaderText(null, false);
					$obj.attr.call($obj, attrName, attrFilter, attrStyle, $val, attrFormat);
				})
			}
			
			return $val;
		}
		else if (property == null) {
			return $val;
		}
		
		if ( !(attrStyle.match(/plaintext/i)) && typeof property!=="string" ) {
			if (attr.match(/FileName|RemoteFile/)) {
				$obj.ExternalFile({
					attribute: attr,
					container: $val,
					embed: attrFormat.match(/embed/i)
				});
				return $val;
			}
		}

        if (typeof property != "object") {
			if (attr.match(/AuditCD|AuditMD/)) {
				$val.addClass("datetime plaintext")
					.append($obj.Date(property, attrFormat));
			}
			else if (attr == "Name" && attrStyle.match(/hyperlink/i)) {
				$val.addClass("hyperlink")
					.append($obj.HyperLink());
			}
			else {
				$val.addClass("plaintext")
					.append(property);
			}
            return;
        }

        if (!property.Value) {
            $val.append(property);
            return;
        }

        $val.addClass(property.Format.toLowerCase());

		switch (property.Format.toLowerCase()) { 
			case "singlelink":
			case "multilink":
				if (attrStyle.match(/repositoryobject/i)) {
					$val = $obj.Links(property, attrFilter, attrStyle, attrFormat);
				}
				else { 
					$val.css("line-height","16px")
						.append($obj.Links(property, attrFilter, attrStyle, null, null, showLinkValue));
					if (!attrFormat) {
						attrFormat = "List";
					}
				}
				break;

			case "singlefile":
			$obj.ExternalFile({
				attribute: attr,
				container: $val,
				value: property.Value,
				embed: attrStyle.match(/embed/i)
			});
			break;

			case "datetime":
				$val.append($obj.Date(property, attrFormat));
				break;

			case "xhtml":
				$val.append($obj.Generic(property, attrStyle));
				break;

			default:
				$val.append($obj.Generic(property, attrStyle));
				break;
		}
    });

    if (attrFormat.match(/list/i) && $val) {
    	$val = $val.toList();
    }
	else if (attrStyle.match(/repositoryobject/i) && $val.jquery){
		if (!$val.text()){
			$val = [];
		}
	}
	 
    return $val;
}

RepositoryObject.prototype.relationattr = function () { return this.relationAttr.apply(this, arguments); }
RepositoryObject.prototype.relationAttr = function () { 
	var $obj = this;
	var $refObj = null;
	var reverse = arguments[1];
	var $val = arguments[2];
	
	if (typeof reverse == "undefined") {
		reverse = false;
	}
	
	if (!$val) {
		$val = $("<span />").addClass("relation")
	}
	
	if (!arguments[0]) {
		return $val;
	}
	
	if (arguments[0] instanceof RepositoryObject) {
		$refObj = arguments[0];
	}
	else if (typeof arguments[0] == "string") {
		$refObj = new RepositoryObject(arguments[0]);
	}
	
	if (!$refObj) {
		return $val.html("-");
	}
	
	if (reverse) {
		$obj = $refObj;
		$refObj = this;
	}
	
	$val.ajaxLoaderText();
	
	$obj.init(function(){
		$val.ajaxLoaderText(false);
		$.each($obj.Data, function(key, content) {
			var relation = key;
			if (typeof content != "object" || !content) {
				return;
			}
			else if( typeof content.Value != "object") {
				return;
			}
			
			$.each(content.Value, function(){
				if (this.ObjId === $refObj.Data.ObjId){
					$val.text() ? 
						$val.append(", " + relation) :
						$val.append(relation);
				}
			});
		});
		
		if (!$val.text()) {
			reverse ? 
				$val.append("LinkedBy") : 
				$val.append("Links");  
		}
	})
	
	return $val;
}

RepositoryObject.prototype.xml = function () { return this.XML.apply(this, arguments); }
RepositoryObject.prototype.XML = function () {
	if (!arguments.length) {
        return;
    }
	
	var attrName = arguments[0];
	
	if (!(attrName in this.Data)) {
		return $();
	}
	
	var xmlstr = this.Data[attrName].Value;
	
	if (xmlstr.indexOf("&#") > -1) {
		var $data = $.parseHTML(this.Data[attrName].Value);
		xmlstr = $($data).get(0);
	}
	
	if (!xmlstr) {
		return;
	}
	
	if (typeof xmlstr != "string") {
		xmlstr = $(xmlstr).html() || xmlstr.nodeValue;
	}
			
	var xml = $.parseXML("<root>" + xmlstr + "</root>");

	return $(xml);
}

RepositoryObject.prototype.Generic = function () {
	if (!arguments.length) {
		return;
	}
	
	var $obj = this;
	var attr = arguments[0] || "";
	var attrStyle = arguments[1] || "";

	var $val = $("<p/>").html(attr.Value ? attr.Value : attr);
	
	if (attrStyle !== "") {
		attrStyle = $.trim(attrStyle).toLowerCase();
	}

	if (attrStyle.match(/plaintext/i)) {
		$val.html($val.text());
		return $val.text();
	}
	else if (attrStyle.match(/hyperlink/i)) {
		var ahref = $val.text();
		var atext = arguments[2] || ahref;
		var $link = $("<a/>")
			.addClass("hyperlink textlink")
			.attr("href", ahref)
			.attr("target", "_blank")
			.html(atext)
			.setTooltip()
			.on($.browser.click, function(e){
				e.preventDefault();
				if (ahref.indexOf(".")>-1 || ahref.substring(0,2)==="\\\\") {
					if (openDocumentFile(ahref)) {
						return false;
					}
				}
				window.open(ahref);
			});
		return $val.html($link);
	}
	else {
		$val.formatText();
		return $val.contents();
	}
}

RepositoryObject.prototype.Date = function () {
	if (!arguments.length) {
		return;
	}

	var $obj = this;
	
	var attr = arguments[0];
	var outputFmt = arguments[1] ? arguments[1] : $QepSettings.DefaultDateFormat;
	var inputFmt = arguments[2] ? arguments[2] : "yyyy MM dd hh.mm:ss";
	var jsFmt = arguments[3] ? arguments[3] : false;
	var jsUTC = arguments[4] ? arguments[4] : false;
	var timestamp = arguments[5] ? arguments[5] : false;

	var d = attr.Value ? attr.Value : attr;
	
	if (!d) {
		return;
	}

	var datestr = "";

	if (d.indexOf("/Date") > -1) {
        	d = eval("new " + d.substring(1, d.length - 1));
		if (jsFmt) {
			datestr = showTime ? d.toLocaleString() : d.toLocaleDateString();
			return datestr;
		}
		else {
			d = d.toJSON();
		}
	}
	
	var yearidx = inputFmt.toLowerCase().indexOf("y");
	var mnthidx = inputFmt.indexOf("M");
	var daysidx = inputFmt.toLowerCase().indexOf("d");
	var houridx = inputFmt.toLowerCase().indexOf("h");
	var mintidx = inputFmt.indexOf("m");
	var scndidx = inputFmt.toLowerCase().indexOf("s");
	
	var yy = mmm = dd = hh = mm = ss = 0;
	
	if (yearidx > -1) {
		if (inputFmt.match("yyyy")) {
			yy = d.substr(yearidx, 4);
		}
		else {
			yy = d.substr(yearidx, 2);
		}
	}
	
	if (mnthidx > -1) {
		mmm = d.substr(mnthidx, 2);
	}
	
	if (daysidx > -1) {
		dd = d.substr(daysidx, 2);
	}
	
	if (houridx > -1) {
		hh = d.substr(houridx, 2);
	}
	
	if (mintidx > -1) {
		mm = d.substr(mintidx, 2);
	}
	
	if (scndidx > -1) {
		ss = d.substr(scndidx, 2);
	}   
	
    if (jsFmt) {
        datestr = new Date(parseInt(yy), parseInt(mmm) - 1, parseInt(dd), parseInt(hh), parseInt(mm), parseInt(ss));
		
		if (jsUTC) {
			datestr = Date.UTC(datestr.getFullYear(), 
				datestr.getMonth(), 
				datestr.getDate(), 
				datestr.getHours(),
				datestr.getMinutes(),
				datestr.getSeconds());
			
			if (!timestamp) {
				datestr = "/Date(" + datestr + ")/";
			}
		}
		
		return datestr;
    }
	else {
		datestr = outputFmt || "yyyy.MM.dd hh.mm:ss";
		if (datestr.match(/yyyy/i)) {
			datestr = datestr.replace(/yyyy/i, yy);
		}
		if (datestr.match(/yy/i)) {
			datestr = datestr.replace(/yy/i, yy.length==4?yy.substr(2,2):yy);
		}
		if (datestr.match("MM")) {
			datestr = datestr.replace("MM", mmm);
		}
		if (datestr.match(/dd/i)) {
			datestr = datestr.replace(/dd/i, dd);
		}
		if (datestr.match(/hh/i)) {
			datestr = datestr.replace(/hh/i, hh);
		}
		if (datestr.match("mm")) {
			datestr = datestr.replace("mm", mm);
		}
		if (datestr.match(/ss/i)) {
			datestr = datestr.replace(/ss/i, ss);
		}
		
		if (d.length < 19) {
			datestr = datestr.substring(0, 16);
		}
		
		if (d.length<16) {
			datestr = datestr.substring(0, 10);
		}
		
	}

    return datestr.replace(/[^A-Za-z 0-9 \.,\?""!@#\$%\^&\*\(\)-_=\+;:<>\/\\\|\}\{\[\]`~]*/g, "");
}

RepositoryObject.prototype.ExternalFile = function() {
	if (!arguments.length) {
        return;
    }

	var $obj = this;
	
	var $opts = {
		attribute: "FileName",
		container: null,
		value: null,
		name: null,
		src: null,
		embed: false
	};
	
	$.extend(true, $opts, arguments[0]);
	
	var $cnt = $opts.container || $("<div/>").addClass("MediaFile");
	var orig = $obj.attr("FileOrigin").text();
	var type = $obj.attr("MediaType").text();
	
	var initFile = function(){	
		if ((type.match(/document/i)
		&& $opts.src.match(/binarydatacontroller.aspx/i))
		|| (!$opts.embed && !type)) {
			var $link = $("<a/>")
				.text($opts.name)
				.attr("href", $opts.src)
				.attr("title", $opts.name)
				.attr("target", "_blank")
				.addClass("hyperlink qlmicon")
				.css({
					"background-image": "url(" + $QisSession.HTMLBase + "/images/templates/document.png)"
				});
				
			var ext = $opts.name.split(".");
			if (ext.length > 1) {
				$link.addClass("qep-icon-16-black-" + ext.pop());
			}
			
			$link.off($.browser.click);
			$link.on($.browser.click, function(e){
				e.preventDefault();
				$link.openDocumentFile($opts.src);
			});
			
			$cnt.append($link);
		}			
		else if ($opts.embed) {
			var $media = $("<div/>").addClass("media-container")
				.css({
					"width": $cnt.findContainerSize("width"),
					"height": $cnt.findContainerSize("height")
				});
				
			$obj.init();
			$media.showMedia({
				src: $opts.src,
				name: $opts.name,
				type: type.toLowerCase(),
				width: $media.width(),
				height: $media.height(),
				autoplay: $("body").data("media-autoplay") || false,
				init: $("body").data("media-init") || $.noop,
				callback: $("body").data("media-callback") || $.noop
			});
			
			$cnt.append($media);
			$.async(function(){
				$cnt.onchange(false);
			}, 100)
		}
		else {
			$link = $obj.Generic($opts.src, "hyperlink", $opts.name).contents();
			$cnt.append($link);
		}
	}
	
	if ($opts.src !== null) {
		if ($opts.src.Value) {
			$opts.src = $opts.src.Value;
		}
		if (!$opts.name) {
			$opts.name = $opts.src;
		}
		
		return initFile();
	}

	switch (parseInt(orig)) {
		case 5:
			if ($opts.value) {
				$opts.src = String($opts.value).toLowerCase().split("src=")[1];
				$opts.src = $opts.src.split("\"")[1];
				if ($opts.src) {
					$opts.src = $QisSession.HTMLBase + "/BinaryDataController.aspx?DataId=" 
						+ $opts.src.replace("qlm:","").replace(/\//g,"") 
						+ "&RepId=" + ($obj.Data.RepId || $QisSession.RepId);
					if (!type.match(/image|video/i)) {
						$opts.src += "&Type=FileLink";
					}
					$opts.name = $obj.attr("RemoteFile", null, "plaintext").text();
				}
				initFile();
			}
			else if ($opts.attribute.match(/RemoteFile/)) {
				$opts.value = $obj.Data.FileAttachment ? $obj.Data.FileAttachment.Value : null;
				if ($opts.value) {
					$opts.src = String($opts.value).toLowerCase().split("src=")[1];
					$opts.src = $opts.src.split("\"")[1];
					$opts.src = $QisSession.HTMLBase + "/BinaryDataController.aspx?DataId=" 
						+ $opts.src.replace("qlm:","").replace(/\//g,"") 
						+ "&RepId=" + ($obj.Data.RepId || $QisSession.RepId);
					if (!type.match(/image|video/i)) {
						$opts.src += "&Type=FileLink";
					}
					$opts.name = $obj.attr("RemoteFile", null, "plaintext").text();
					initFile();
				}
			}
			break;
		case 6:
			if ($opts.attribute.match(/FileName/)) {
				return $obj.info("FileName", function() {
					$opts.src = $obj.attr("FileName", null, "plaintext").text();
					$opts.name = $obj.attr("RemoteFile", null, "plaintext").text();
					initFile();
				}, "ServiceHandlers.GetSharePointLink", false);
			}
		
		default: 
			if ($opts.attribute.match(/FileName/)) {
				$opts.src = $obj.attr("FileName", null, "plaintext").text();
				$opts.name = $opts.src;
				initFile();
			}
			break;
	}

	if (!$opts.container) {
		return $cnt;
	}
}

RepositoryObject.prototype.Links = function () {
    if (!arguments.length) {
        return;
    }

    var $obj = this;
    var attr = arguments[0];

    var templateFilter = arguments[1] ? arguments[1] : "";
    var attrStyle = arguments[2] ? arguments[2] : "";
    var linkImage = arguments[3] ? arguments[3] : $QepSettings.Definitions.ListViewImage;
    var attrFormat = arguments[4] ? arguments[4] : "";
    var showLinkValue = arguments[5] ? arguments[5] : false;
	
    var asRepObject = false;
	
	if (attrStyle!=="") {
		attrStyle = $.trim(attrStyle).toLowerCase();
	}

	if (attrFormat!=="") {
		attrFormat = $.trim(attrFormat).toLowerCase();
	}

    if (attrStyle.match(/repositoryobject/i)) {
        asRepObject = true;
    }

    var $val = asRepObject ? [] : $("<p/>");

    $(attr.Value).each(function () {
		var $obj1 = new RepositoryObject(this);
		
        if (templateFilter.length) {
            if ($.inArray($obj1.Data.Template, templateFilter) < 0) {
                return;
            }
        }
        
		if (asRepObject) {
			$val.push($obj1);
		}
		else if (attrStyle.match(/plaintext/i)) {
			$val.append($("<div/>").text($obj1.Data.Name));
		}
		else {
			$val.tableOp($obj1.HyperLink(null, null, linkImage, null, null, showLinkValue), attrFormat);
		}
    });

    return asRepObject 
		? $val 
		: (attrStyle.match(/plaintext/i)
			? $val.html() 
			: $val.children());
}

RepositoryObject.prototype.HyperLink = function () {
    var $obj = arguments[0] ? arguments[0] : this;
    var style = arguments[1] ? arguments[1] : "";
    var linkImage = $QepSettings.Definitions.ListViewImage;
    var imagePos = arguments[3] ? arguments[3] : "";
	var showLinkValue = arguments[5] ? arguments[5] : false;
	
    var tooltip = true;
	
	if ( typeof arguments[2] != "undefined" ) {
		if (arguments[0] != null) {
			linkImage = arguments[2];
		}
	}
	
	if ( typeof arguments[4] == "undefined" ) {
		if (arguments[4] != null) {
			tooltip = arguments[4];
		}
	}

	if (!($obj instanceof RepositoryObject)){
		$obj = new RepositoryObject($obj);
	}
	
    var onm = $obj.Data.Name;
    var oid = $obj.Data.ObjId;
    var tmp = $obj.Data.Template || "";
    var rev = $obj.Data.RevisionId || $obj.Data.RevId;
    var lcd = $obj.Data.Language || $QisSession.LangCode;
    var cid = $obj.Data.ConfId || $QisSession.ConfId;
    var rid = $obj.Data.RepId || $QisSession.RepId;

	var info = getRepositoryInfo(rid);
	var target = info.Target;
	
	if (tmp.match(/EmptyStub/i)) {
		style = "PlainText";
		tmp = "";
	}
	else if (!info.FollowLinks || !info.WebApplication) {
		style = "PlainText";
	}
		
	if (showLinkValue && $.keyCount($obj.Data.LinkValues)) {
		var linkvalue = "";
		linkvalue += " [";
		$.each($obj.Data.LinkValues, function(index, value){
			if (index) {
				linkvalue += ",";
			}
			linkvalue += value;
		});
		linkvalue += "]";
		if ($.trim(linkvalue) != "[]") {
			onm += linkvalue;
		}
	}
	
	var $anch = $("<a/>")
		.addClass("hyperlink")
		.attr("title", onm)
		.text(onm)
		.data({
			objid: oid,
			revisionid: rev,
			template: tmp,
			name: onm,
			language: lcd,
			confid: cid,
			repid: rid
		});
	
	if (style.match(/plaintext/i)) {
		$anch.disableHyperLink()
		if (tmp) {
			$anch.setIcon();
		}
		else {
			$anch.css({
				"padding-left": "20px",
				"line-height": "16px",
				"display": "inline-block"
			})
		}
	}
	else {
		var href = $QisSession.HTMLLocation
			+ (oid ? "?oid=" + oid : "");
		
		if (lcd!==$QisSession.DefaultLangCode) {
			href += "&lc=" + lcd;
		}
		
		if (cid!==$QisSession.BaseConfId) {
			href += "&cid=" + cid;
		}
		
		if (rid!==$QisSession.RepId) {
			if (!target.match(/blank|self/i)) {
				href += "&rid=" + rid;
			}
			else {
				href = href.replace(location.pathname, "/" + info.WebApplication + "/");
			}
		}
		
		$anch.attr("href", href);
		
		if (linkImage) {
			$.async(function(){
				$anch.setIcon(imagePos);
			});
		}
		
		if (target.match(/blank|self/i)) {
			$anch.attr("target", target);
			if (tooltip) {
				$anch.setTooltip();
			}
		}
		else {
			$anch.setTarget(tooltip);
		}
	}

	return $anch;
}

RepositoryObject.prototype.BFSLinks = function () {
    if (!arguments.length) {
        return;
    }

    var $obj = this;

    var attrName = arguments[0] ? arguments[0] : [];
    var attrFilter = arguments[1] ? arguments[1] : [];
    var relName = arguments[2] ? arguments[2] : [];
    var relFilter = arguments[3] ? arguments[3] : [];
    var maxlvl = arguments[4] ? arguments[4] : 0;
    var attrStyle = arguments[5] ? arguments[5] : "";
    var attrFormat = arguments[6] ? arguments[6] : "";
    var linkImage = arguments[7] ? arguments[7] : $QepSettings.Definitions.ListViewImage;
    var currlvl = arguments[8] ? arguments[8] : 0;
    var $val = arguments[9] ? arguments[9] : $("<div/>");
    var callback = arguments[10] ? arguments[10] : {};
    var $counter = arguments[11] ? arguments[11] : $("<div/>").data("index", 0).data("count", 0).data("blist", "").data("plist", "");

	var clbfunc = {
		onStart: $.noop,
		onInit: $.noop,
		onFinish: $.noop,
		onObject: $.noop
	}
	
	callback = $.extend(clbfunc, callback);
	
	if (attrStyle!=="") {
		attrStyle = $.trim(attrStyle).toLowerCase();
	}

	if (attrFormat!=="") {
		attrFormat = $.trim(attrFormat).toLowerCase();
	}
	
	if (arguments[9] === false) {
		$val = false;
	}
	else if (attrName == "") {
		return $val.html();
	}
	
	if (!currlvl && $val) {
		callback.onStart.call($obj, $val);
	}
	
    if (typeof relName === "string") {
        relName = relName.replace(/;/g,",").split(",");
    }

    if (typeof relFilter === "string") {
        relFilter = relFilter.replace(/;/g,",").split(",");
    }

    if (typeof attrName === "string") {
        attrName = attrName.replace(/;/g,",").split(",");
    }

    if (typeof attrFilter === "string") {
        attrFilter = attrFilter.replace(/;/g,",").split(",");
    }
	
	var $links = $obj.attr(attrName.diff($obj.genericAttributes()), attrFilter, attrStyle).find("a");
	
	$links.each(function() {
		if ($counter.data("blist").indexOf($(this).data("objid")) < 0) {
			$counter.data("blist", $counter.data("blist") + ";" + $(this).data("objid"));
			if ($val) {
				$val.tableOp($(this), attrFormat, true);
			}
			callback.onObject.call($(this), $(this));
		}
	});

    if (currlvl < maxlvl) {
        $.each(relName, function (idx, relation) {
            $obj.BFSLinksNextLevel(attrName, attrFilter, relName, relFilter, maxlvl, attrStyle, attrFormat, linkImage, currlvl + 1, $val, callback, $counter, relation);
        });
    }
	
	if (!currlvl) {
		$.async(function(){
			if (!$counter.data("index")) {
				$.async(false, this);
				callback.onFinish.call($obj, $val);
			}
		}, 250, true);
	}
	
	if (!currlvl && $val) {
        $val.addClass("bfs links");
        return $val;
    }
}

RepositoryObject.prototype.BFSLinksNextLevel = function () {
    if (!arguments.length) {
        return;
    }

    var $obj = this;

    var attrName = arguments[0] ? arguments[0] : [];
    var attrFilter = arguments[1] ? arguments[1] : [];
    var relName = arguments[2] ? arguments[2] : [];
    var relFilter = arguments[3] ? arguments[3] : [];
    var maxlvl = arguments[4] ? arguments[4] : 0;
    var attrStyle = arguments[5] ? arguments[5] : "";
    var attrFormat = arguments[6] ? arguments[6] : "";
    var linkImage = arguments[7] ? arguments[7] : $QepSettings.Definitions.ListViewImage;
    var currlvl = arguments[8] ? arguments[8] : 0;
    var $val = arguments[9] ? arguments[9] : "";
    var callback = arguments[10] ? arguments[10] : { onObject: $.noop, onInit: $.noop, onFinish: $.noop };
    var $counter = arguments[11] ? arguments[11] : $("<div/>").data("index", 0).data("count", 0).data("blist", "").data("plist", "");
    var currRelName = arguments[12] ? arguments[12] : "";

	if (arguments[9] === false) {
		$val = false;
	}
	
    if (!currRelName) {
        return;
    }
	
	var BFSLinksProcess = function (data) {
		if (!isValidJsonData.call($obj, data, "WebQuery")) {
			data = "";
		}
		
		var $data = data.length ? $.parseJSON(data) : {};
		$.each($data, function (index, data) {
			var $obj1 = new RepositoryObject(data);
			var count = $counter.data("count") + 1 ;
			callback.onInit.call($obj1, attrName, currlvl, count);
			$counter.data("count", count);
			
			if (isContainsRelation && ($.inArray("Contains", attrName)>-1 || $.inArray("ContainedIn", attrName)>-1)) {
				if (!attrFilter.length || $.inArray($obj1.Data.Template, attrFilter)>-1) {
					if ($counter.data("blist").indexOf($obj1.Data.ObjId) < 0) {
						$counter.data("blist", $counter.data("blist") + ";" + $obj1.Data.ObjId);
						var $link = $obj1.HyperLink();
						if ($val) {
							$val.tableOp($link, attrFormat, true);
						}
						callback.onObject.call($link, $link);
					}
				}
			}
			
			if (relFilter.length && $.inArray($obj1.Data.Template, relFilter) < 0) {
				return;
			}
			
			if ($counter.data("plist").indexOf($obj1.Data.ObjId) > -1
			|| $obj1.Data.Template.match(/EmptyStub/i)
			|| !getRepositoryInfo($obj1.Data.RepId).FollowLinks ) {
				return;
			}
			
			$counter
				.data("plist", $counter.data("plist") + ";" + $obj1.Data.ObjId)
				.data("index", $counter.data("index") + 1);
			
			$obj1.init(function(){
				$obj1.BFSLinks(attrName, attrFilter, relName, relFilter, maxlvl, attrStyle, attrFormat, linkImage, currlvl, $val, callback, $counter);
				$counter.data("index", $counter.data("index") - 1);
			});
		});
	}

	$counter.data("index", $counter.data("index") + 1);

	var isContainsRelation = currRelName == "Contains" ? true : false;
	var isGenericRelation = ($.inArray(currRelName, ["Links", "Contains"]) > -1) ? true : false;

	var wsurl = initQisWebService({
		objid: $obj.Data.ObjId,
		revid: $obj.Data.RevisionId,
		confid: $obj.Data.ConfId,
		repid: $obj.Data.RepId,
		relation: isContainsRelation ? "Contains" : "LinksTo", 
		field: (isGenericRelation) ? "" : currRelName
	});
	
	if ($.pageCache("get", wsurl, true)) {
		var data = $.pageCache("get", wsurl);
		BFSLinksProcess(data);
		$counter.data("index", $counter.data("index") - 1);
	}
	else if ($.pageCache("init", wsurl)) {	
		$.async( function(){
			if ($.pageCache("init", wsurl)) {
				return;
			}
			$.async(false, this);
			var data = $.pageCache("get", wsurl);
			BFSLinksProcess(data);
			$counter.data("index", $counter.data("index") - 1);
		}, 150, true);
	}
	else {
		$.pageCache("init", wsurl, true);
		$.browser.log("BFS: attrbutes="+attrName+"; filter="+attrFilter+"; relations="+relName+"; filter="+relFilter+"; max-level="+maxlvl+"; current-level="+currlvl);
		$.getJSON(wsurl)
			.done( function (data){
				$.pageCache("set", wsurl, data);
				BFSLinksProcess(data);
				$counter.data("index", $counter.data("index") - 1);
			})
			.fail( function(){
				$.pageCache("set", wsurl, "{}");
				$counter.data("index", $counter.data("index") - 1);
			})
	}
}

RepositoryObject.prototype.RelationQuery = function () { return this.GenericRelation.apply(this, arguments); }
RepositoryObject.prototype.GenericRelation = function () {
	var $obj = this;

    var $opts = {
        relation: "LinksTo",
        attribute: null,
        templateFilter: [],
        sortOrder: "asc",
        sortAttribute: "Name",
        callback: null
    }

    if (arguments.length) {
        $opts = $.extend($opts, arguments[0]);
        $opts.callback = arguments[0].callback;
		if ($opts.relation == "Links") { 
			$opts.relation = "LinksTo";
		}
    }
    
    if (!$opts.callback) {
        $.browser.warn("GenericRelation: callback function is not defined");
        return;
    }

    if (typeof templateFilter == "string") {
        templateFilter = templateFilter.split(",");
    }

    var revisionId = null;
 
    if ($opts.relation.match(/LinksTo|Contains/i)) {
        revisionId = $obj.Data.RevisionId;
        if (!revisionId) {
            return $obj.audit(function () {
                $obj.GenericRelation.call($obj, $opts);
            });
        }
    }
	
	var buildQueryLinks = function(data){
		if (!isValidJsonData.call($obj, data, "GenericRelation")) {
			data = "";
		}
		
		var $data = data.length ? $.parseJSON(data) : {};
		var $objlist = new Array();
		$.each($data, function () {
			var $obj1 = new RepositoryObject(this);
			if (!isObjectExcluded($obj1, $opts.templateFilter)) {
				$objlist.push($obj1);
			}
		});

		$objlist = $obj.sortObjectList($objlist, {
		   sortAttribute: $opts.sortAttribute,
		   sortOrder: $opts.sortOrder
		});

		$opts.callback.call($objlist);
	}
	
	var wsurl = initQisWebService({
		objid: $obj.Data.ObjId, 
		revid: revisionId, 
		confid: $obj.Data.ConfId,
		repid: $obj.Data.RepId,
		relation: $opts.relation, 
		field: $opts.attribute
	});
	
	if ($.pageCache("get", wsurl, true)) {
		var data = $.pageCache("get", wsurl);
		buildQueryLinks(data);
	}
	else if ($.pageCache("init", wsurl)) {	
		$.async( function(){
			if ($.pageCache("init", wsurl)) {
				return;
			}
			$.async(false, this);
			var data = $.pageCache("get", wsurl);
			buildQueryLinks(data);
		}, 150, true);
	}
	else {
		$.browser.log("GenericRelation: relation="+$opts.relation+" attribute="+$opts.attribute);
		$.pageCache("init", wsurl, true);
		$.getJSON(wsurl)
			.done( function (data){
				$.pageCache("set", wsurl, data);
				buildQueryLinks(data);
			})
			.fail( function(){
				$opts.callback.call([]);
				$.pageCache("set", wsurl, "{}");
			})
	}
}

RepositoryObject.prototype.GraphRelation = function () {
	var $obj = this;

    var $opts = {
        relation: "Frames",
        templateFilter: [],
	diagram: null,
        sortOrder: "asc",
        sortAttribute: "Name",
        callback: null
    }

    if (arguments.length) {
        $opts = $.extend($opts, arguments[0]);
        $opts.callback = arguments[0].callback;
    }
	
    if (!$opts.callback) {
        $.browser.warn("GraphQuery: callback function is not defined");
        return;
    }

    if (typeof templateFilter == "string") {
		if (templateFilter) {
			templateFilter = templateFilter.split(",");
		}
    }
	
	if ($opts.diagram instanceof RepositoryObject) {
		$opts.diagram = $opts.diagram.Data.ObjId;		
	}
 
	var buildQueryLinks = function(data){
		if (!isValidJsonData.call($obj, data, "GraphRelation")) {
			data = "";
		}
		
		var $data = data.length ? $.parseJSON(data) : {};
		var $objlist = new Array();
		$.each($data, function () {
			var $obj1 = new RepositoryObject(this);
			if (!isObjectExcluded($obj1, $opts.templateFilter)) {
				$objlist.push($obj1);
			}
		});

		$objlist = $obj.sortObjectList($objlist, {
		   sortAttribute: $opts.sortAttribute,
		   sortOrder: $opts.sortOrder
		});

		$opts.callback.call($objlist);
	}
	
	var url = initQisCsharpMethod.call($obj, "ServiceHandlers.GetGraphicalRelation");
	var $parms = { 
		objectAttributes:[
			{Name:"ObjId", Format:"PlainText", Value: $obj.Data.ObjId},
			{Name:"Relation", Format:"PlainText", Value: $opts.relation}
		]};
	
	if ($opts.diagram) {
		$parms.objectAttributes.push({Name:"DiagramId", Format:"PlainText", Value: $opts.diagram});
	}
	
	var wsurl = window.JSON.stringify($parms) + "_"  + url;
	
	if ($.pageCache("get", wsurl, true)) {
		var data = $.pageCache("get", wsurl);
		buildQueryLinks(data);
	}
	else if ($.pageCache("init", wsurl)) {	
		$.async( function(){
			if ($.pageCache("init", wsurl)) {
				return;
			}
			$.async(false, this);
			var data = $.pageCache("get", wsurl);
			buildQueryLinks(data);
		}, 150, true);
	}
	else {
		$.browser.log("GraphQuery: relation="+$opts.relation+" diagram="+$opts.diagram);
		$.pageCache("init", wsurl, true);
		$.ajax({
			type: 'POST',
			dataType: 'JSON',
			contentType: 'application/json',
			url: url,
			data: window.JSON.stringify($parms)
		})
		.done( function (data){
			$.pageCache("set", wsurl, data);
			buildQueryLinks(data);
		})
		.fail( function(){
			$opts.callback.call([]);
			$.pageCache("set", wsurl, "{}");
		})
	}
}

RepositoryObject.prototype.GenericQuery = function () {
	var $obj = this;

    var $opts = {
		method: "findobjects",
        templates: [],
		filter: [],
		filterType: "And",
		detectMetamodelFormat: false,
		sortAttribute: "Name",
		sortOrder: "Ascending",
		top: 10,
		skip: 0,
		callback: $.noop
    }

    if (arguments.length) {
        $opts = $.extend($opts, arguments[0]);
        $opts.callback = arguments[0].callback;
		if (!$opts.templates) {
			$opts.templates = [];
		}
    }
    
	if (!$opts.callback) {
		$.browser.warn("GenericQuery: callback function is not defined");
		return;
	}
	
	var custom = false;
	
	if ($opts.method.match(/findname/i)) {
		$opts.method = "findobjects";
	}
	
	if (!$opts.method.match(/findobjects/i)) {
		custom = true;
	}
	
	if (!$opts.templates.length && !custom) {
		$opts.callback.call([]);
	}
	
	if (typeof $opts.templates == "string") {
        	$opts.templates = $opts.templates.split(/,|;/);
	}
	else if (!$opts.templates) {
		$opts.templates = [];
	}
	
	if (!$opts.templates.length  && !custom) {
		return $opts.callback.call({});
	}
	
	switch ($opts.sortAttribute) {
		case "AuditCD":
			$opts.sortAttribute = "sys_Created";
			break;
		case "AuditCN":
			$opts.sortAttribute = "sys_CreatedBy";
			break;
		case "AuditMD":
			$opts.sortAttribute = "sys_Modified";
			break;
		case "AuditMD":
			$opts.sortAttribute = "sys_ModifiedBy";
			break;
		default: break;
	}
	
	var url = "";
	if (custom){
		url = initQisCsharpMethod.call($obj, $opts.method);
	}
	else {
		url = initQisCsharpMethod.call($obj, $opts.method, null);
	}
	var $parms = {};
	if (custom) {
		$parms = {
			objectAttributes:[								
				{Name:"ObjId", Format:"PlainText", Value: $obj.Data.ObjId},
				{Name:"RevisionId", Format:"PlainText", Value: $obj.Data.RevisionId},
				{Name:"Name", Format:"PlainText", Value: $obj.Data.Name}
			]
		};
		if (arguments[0]) {
			$.each(arguments[0], function(key, val){
				if (key === "callback") { return true; }
				$parms.objectAttributes.push({
					Name: key,
					Format: "PlainText",
					Value: val
				});
			});
		}
	}
	else {
		$parms = {
			templates : $opts.templates,
			filter: $opts.filter ? $opts.filter : [],
			filterType: $opts.filterType ? $opts.filterType : "And",
			detectMetamodelFormat: $opts.detectMetamodelFormat ? $opts.detectMetamodelFormat : false,
			order: [{
				key: $opts.sortAttribute ? $opts.sortAttribute : "Name", 
				value: $opts.sortOrder ? ($opts.sortOrder.match(/asc/i) ? "asc" : "desc") : "asc"
			}],
			top: $opts.top,
			skip: $opts.skip
		};
		
	}

	$parms = window.JSON.stringify($parms);
		
	var buildQueryLinks = function(data){
		if (!isValidJsonData.call($obj, data, $opts.method)) {
			data = "";
		}
		
		var $data = data.length ? $.parseJSON(data) : {};
		var $objlist = new Array();
		$.each($data, function () {
			var $obj1 = new RepositoryObject(this);
			$objlist.push($obj1);
		});

		$opts.callback.call($objlist);
	}
	
	var wsurl = $parms + "_"  + url;

	if ($.pageCache("get", wsurl, true)) {
		var data = $.pageCache("get", wsurl);
		buildQueryLinks(data);
	}
	else if ($.pageCache("init", wsurl)) {	
		$.async( function(){
			if ($.pageCache("init", wsurl)) {
				return;
			}
			$.async(false, interval);
			var data = $.pageCache("get", wsurl);
			buildQueryLinks(data);
		}, 150, true);
	}
	else {
		$.browser.log("Query: type=findobjects" + ", parms=" + $opts);
		$.pageCache("init", wsurl, true);
		$.ajax({
			type: 'POST',
			dataType: 'JSON',
			contentType : 'application/json',
			url: url,
			data: $parms
		})
		.done( function (data){
			$.pageCache("set", wsurl, data);
			buildQueryLinks(data);
		})
		.fail( function(jqxhr, textStatus, error){
			notifyRequestError(jqxhr, textStatus, error, url, $parms, null);
			$opts.callback.call([]);
			$.pageCache("set", wsurl, "{}");
		})
	}
}

RepositoryObject.prototype.UserSubscriptions = function () {
	var $obj = this;
	
	var format = arguments[3] || "list";
	if (format.match(/treeview/i)) {
		return $obj.UserSubscriptionsTree.apply($obj, arguments);
	}
	
	var $list = $("div.subscriptions");

	var cache = typeof arguments[0] != "undefined" ? arguments[0] : true;
	var callback = arguments[1] ? arguments[1] : $.noop;
	var clbParm = arguments[2] ? arguments[2] : null;
	var emptytxt = arguments[4] ? arguments[4] : "";

	if (!$list.length) {
		$list = $("<div/>").addClass("subscriptions links");
	}
	else {
		var $cnt = $list.parents(".ui-panel-content-text");
		$list.detach().appendTo($cnt.empty());
		if ($list.hasClass("treeview") || $list.find(".ui-fancytree").length) {
			return $obj.UserSubscriptionsTree.apply($obj, arguments);
		}
	}
           
    $list
        .empty()
        .ajaxLoaderText();
		
	var buildUserSubscriptions = function(data) {
		if (!isValidJsonData.call(null, data, "GetUserSubscriptions")) {
			data = "";
		}
		
		var $data = data.length ? $.parseJSON(data) : {};
		if (!$.keyCount($data) && !$data.length) {
			$list.html(emptytxt);
			callback.call($list, clbParm);
			return;
		}

		var $objlist = new Array(); 
		$.each($data, function(){
			var $obj1 = new RepositoryObject(this);
			$objlist.push($obj1.HyperLink());
		});
						
		if ($objlist.length) {
		   $list.html($objlist).toList();
		}
		else {
			$list.html(emptytxt);
		}
		
		callback.call($list, clbParm);
	}

	var data = cache ? 
		$.browserCache("get", "Subscriptions" + $QisSession.ConfId) :
		null;
	
	if (data) {
		buildUserSubscriptions(data, true);
	}
	else {
		var wsurl = initQisCsharpMethod.call(null, "ServiceHandlers.GetUserSubscriptions");
		
		$.ajax({
			type: 'POST',
			dataType: 'JSON',
			contentType : 'application/json',
			url: wsurl,
			data: window.JSON.stringify({ objectAttributes: [] })
		})
		.done(function (data) {
			$.browserCache("set", "Subscriptions" + $QisSession.ConfId, data);
			buildUserSubscriptions(data, false);
		})
		.fail(function(jqxhr, textStatus, error){
			notifyRequestError(jqxhr, textStatus, error, wsurl, null, null);
			$.browser.error("Failed getting subscriptions")
			$list.html(emptytxt);
		});
	}

    return $list;
}

RepositoryObject.prototype.UserSubscriptionsTree = function () {
	var $obj = this;
	
	var cache = typeof arguments[0] != "undefined" ? arguments[0] : true;
	var callback = arguments[1] ? arguments[1] : $.noop;
	var clbParm = arguments[2] ? arguments[2] : null;
	var emptytxt = arguments[4] ? arguments[4] : "";
	
	var $tree = $("div.subscriptions");
	
	if (!$tree.length) {
		$tree = $("<div/>").addClass("subscriptions links treeview");
	}
	else {
		$tree.fancytree().fancytree("destroy");
	}
	
	$tree
		.empty()
		.ajaxLoaderText();

	var wsurl = initQisCsharpMethod.call(null, "ServiceHandlers.GetUserSubscriptionsTree");

	var buildTreeView = function() {
		var data = arguments.length ? arguments[0] : null;
		$tree.fancytree({
			source: data,
			debugLevel: 0,
			clickFolderMode: 3,
			lazyLoad: function(event, data) {
				var node = data.node;				
				var parms = [];
				
				var odata = node.data;
				
				if ($.isEmptyObject(odata)) {
					return;
				}
					
				if (odata.Type) {
					parms.push({Name: "SubscriberType", Format: "PlainText", Value: odata.Type})
				}
				else {
					parms.push({Name: "SubscriberId", Format: "PlainText", Value: odata.ObjId})
				}
				
				data.result = {
					type: 'POST',
					dataType: 'JSON',
					contentType : 'application/json',
					url: wsurl,
					data: window.JSON.stringify({ 
						objectAttributes: parms
					})
				}
			},
			createNode: function(event, data){
				var node = data.node;
				var $data = node.data
				var $el = $(node.li);
				
				if (!$data.Total) {
					var $obj = new RepositoryObject($data);
					var $lnk = $obj.HyperLink();
					$el.addClass("fancytree-child")
						.find(".fancytree-title")
							.html($obj.HyperLink(null, null, null, null, false).setTooltip())
							.siblings().remove();
				}
				else {
					var $icon = $el.find(".fancytree-icon")
					.css({
						"background-image": "url(" + $QisSession.HTMLBase + "/images/templates/" + $data.Template + ".png)",
						"background-position": "0 0"
					});
				
					if ($data.Total) {
						$icon.append($("<span/>")
							.addClass("fancytree-childcounter")
							.html($data.Total));
					}
				}
			},
			postProcess: function(event, data) {
				data.result = [];
				
				var $content = data.response;
				if ($content[0] && $.isEmptyObject($content) ) {
					return;
				}
				
				var isteam = $content.Subscribers;
				var $itms = isteam ? $content.Subscribers : $content.Subscribed;
				
				if ($itms[0] && $.isEmptyObject($itms) ) {
					return;
				}
				
				$.each($itms, function(){
					var title, odata = {}, num = 0;
					
					if (isteam) {
						num = this.Total;
					}
					
					title = this.Name;
					odata = this;
					
					data.result.push({
						title: title,
						data: odata,
						lazy: num ? true : false,
						folder: num ? true : false
					})
				});
			},
			collapse: function() {
				var data = {Content: $tree.fancytree("getTree").toDict(), Total: $tree.data("counter")};
				$.browserCache("set", "Subscriptions" + $QisSession.ConfId, window.JSON.stringify(data));
			},
			expand: function() {
				var data = {Content: $tree.fancytree("getTree").toDict(), Total: $tree.data("counter")};
				$.browserCache("set", "Subscriptions" + $QisSession.ConfId, window.JSON.stringify(data));
			}
		});

		$tree.ajaxLoaderText(false);
		callback.call($tree, clbParm);
	}
		
	if (cache) {
		var $data = $.browserCache("get", "Subscriptions" + $QisSession.ConfId);
		if ($data) {
			$data = $.parseJSON($data);
			if ($data.Content.length) {
				buildTreeView($data.Content);
				$tree.data("counter", $data.Total).data("domevent", false);
				return $tree;
			}
		}
	}
  
	$.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		url: wsurl,
		data: window.JSON.stringify({objectAttributes: []})
	})
	.done(function (data) {
		if (!isValidJsonData.call(null, data, "GetUserSubscriptionsTree")) {
			data = "";
		}
		
		if (!data.length) {
			return $tree.empty();
		}
		
		var $data = $.parseJSON(data);		
		if (!$data.Total) {
			$tree.html(emptytxt);
			callback.call($tree, clbParm);
			return;
		}
		
		$tree.data("counter", $data.UniqueTotal).data("domevent", false);
		
		var $ul = $("<ul/>")
			.appendTo($tree);
			
		$.each($data.SubscriptionTypes, function(){
			var typ = this.Type;
			var num = this.Total;
			var unq = this.UniqueTotal;
			var tmp = typ.match(/personal/i) ? "Person" : "OrganizationUnit"
			$ul.append($("<li/>")
				.addClass("folder lazy")
				.data("json", {
					Template: tmp,
					Type: typ,
					Total: num,
					Unique: unq
				})
				.text($.LL(typ)));
		});

		buildTreeView();
		
		var data = {Content: $tree.fancytree("getTree").toDict(), Total: $tree.data("counter")};
		$.browserCache("set", "Subscriptions" + $QisSession.ConfId, window.JSON.stringify(data));
	})
	.fail(function(jqxhr, textStatus, error){
		notifyRequestError(jqxhr, textStatus, error, wsurl, null, null);
		$tree.html(emptytxt);
		callback.call($tree, clbParm);
		return;
	})
	
	return $tree;
}

RepositoryObject.prototype.GovernanceTasks = function () {
	var $obj = this;
	
	var format = arguments[3] || "list";
	if (format.match(/treeview/i)) {
		return $obj.GovernanceTasksTree.apply($obj, arguments);
	}

	var $list = $("div.governancetasks");
	
	var cache = typeof arguments[0] != "undefined" ? arguments[0] : true;
	var callback = arguments[1] ? arguments[1] : $.noop;
	var clbParm = arguments[2] ? arguments[2] : null;
	var emptytxt = arguments[4] ? arguments[4] : "";
	
	if (!$list.length) {
		$list = $("<div />").addClass("governancetasks links");
	}
	else {
		var $cnt = $list.parents(".ui-panel-content-text");
		$list.detach().appendTo($cnt.empty());
		if($list.hasClass("treeview") || $list.find(".ui-fancytree").length) {
			return $obj.GovernanceTasksTree.apply($obj, arguments);
		}
	}
	
    $list
        .empty()
        .ajaxLoaderText();
	
	var buildTasksList = function(data) {
		if (!isValidJsonData.call(null, data, "GetGovernanceTasks")) {
			data = "";
		}
		
		var $data = data.length ? $.parseJSON(data) : {};
		if (!$.keyCount($data) && !$data.length) {
			$list.html(emptytxt);
			callback.call($list, clbParm);
			return;
		} 
		
		var fldWidth = $list.width() || ($QepSettings.Definitions.FieldLeft.Width - 20);
		var fontSize = 6.75;
		var chLength = parseInt(fldWidth) / fontSize;
		
		var $objlist = new Array(); 

		$.each($data, function () {
			var $obj = new RepositoryObject(this);
			
			var mode = $obj.Data.Behavior || $obj.Data.behaviour;
			if (mode.match(/sleeping|idle/i)) {
				return;
			}

			var $anch = $obj.HyperLink();
			
			$anch
				.setGovernanceTaskTooltip($obj)
				.restrictTextLength(Math.floor(chLength))
				.setGovernanceTaskIndicator($obj);
				
			$objlist.push($anch);
		});
		
		if ($objlist.length) {
		   $list.html($objlist).toList();
		}
		else {
			$list.html(emptytxt);
		}
		
		callback.call($list, clbParm);
	}
	
	var data = cache ? 	
		$.browserCache("get", "Tasks" + $QisSession.RepId) :
		null;
	
	if ( data ) {
		buildTasksList(data);
	}
	else {
		var ConfIdList = "";
		$.each($ConfigurationInfo.Data||{}, function(idx, data){
			ConfIdList += (ConfIdList ? "," : "") + data.Id;
		});
		
		var wsurl = initQisCsharpMethod.call(null, "GWE_ServiceHandlers.GetGovernanceTasks");
		
		$.ajax({
			type: 'POST',
			dataType: 'JSON',
			contentType : 'application/json',
			url: wsurl,
			data: window.JSON.stringify({ 
				objectAttributes: [
					{Name:"ConfigurationFilter",Format:"PlainText",Value:ConfIdList}
				]
			})
		})
		.done(function (data) {
			$.browserCache("set", "Tasks" + $QisSession.RepId, data);		
			buildTasksList(data);
		})
		.fail(function(jqxhr, textStatus, error){
			notifyRequestError(jqxhr, textStatus, error, wsurl, null, null);
			$.browser.error("Failed getting governance tasks")
			$list.html(emptytxt);
		})
	}

	return $list;
}

RepositoryObject.prototype.GovernanceTasksTree = function () {
	var $obj = this;
	
	var cache = typeof arguments[0] != "undefined" ? arguments[0] : true;
	var callback = arguments[1] ? arguments[1] : $.noop;
	var clbParm = arguments[2] ? arguments[2] : null;
	var emptytxt = arguments[4] ? arguments[4] : "";
	
	var ConfIdList = "";
	$.each($ConfigurationInfo.Data||{}, function(idx, data){
		ConfIdList += (ConfIdList ? "," : "") + data.Id;
	});
	
	
	var $tree = $("div.governancetasks");
	
	if (!$tree.length) {
		$tree = $("<div/>").addClass("governancetasks links treeview");
	}
	else {
		$tree.fancytree().fancytree("destroy");
	}
	
	$tree
		.empty()
		.ajaxLoaderText();
	
	var wsurl = initQisCsharpMethod.call(null, "GWE_ServiceHandlers.GetGovernanceTasksTree");

	var buildTreeView = function() {
		var data = arguments.length ? arguments[0] : null;
		$tree.fancytree({
			source: data,
			debugLevel: 0,
			clickFolderMode: 3,
			lazyLoad: function(event, data) {
				var node = data.node;				
				var parms = [];
				
				var odata = node.data;
				var pdata = node.parent.data;
				
				if (!$.isEmptyObject(pdata)) {
					parms.push({Name: pdata.Template, Format: "PlainText", Value: pdata.ObjId})
				}
				
				if (!$.isEmptyObject(odata)) {
					parms.push({Name: odata.Template, Format: "PlainText", Value: odata.ObjId})
				}
				
				data.result = {
					type: 'POST',
					dataType: 'JSON',
					contentType : 'application/json',
					url: wsurl,
					data: window.JSON.stringify({ 
						objectAttributes: parms
					})
				}
			},
			createNode: function(event, data){
				var node = data.node;
				var $data = node.data
				var $el = $(node.li);
				
				if ($data.Status) {
					var $obj = new RepositoryObject($data);
					
					var $lnk = $obj.HyperLink();
					$el.addClass("fancytree-child")
						.find(".fancytree-title")
							.html($obj.HyperLink(null, null, null, null, false).setTooltip())
							.siblings().remove();
				}
				else {
					var $icon = $el.find(".fancytree-icon")
					.css({
						"background-image": "url(" + $QisSession.HTMLBase + "/images/templates/" + $data.Template + ".png)",
						"background-position": "0 0"
					});
				
					if ($data.Total) {
						$icon.append($("<span/>")
							.addClass("fancytree-childcounter")
							.html($data.Total));
					}
				}
			},
			postProcess: function(event, data) {
				data.result = [];
				
				var $content = data.response;
				if ($content[0] && $.isEmptyObject($content) ) {
					return;
				}
				
				var $tasks = $content.GovernanceTasks;
				if ($tasks[0] && $.isEmptyObject($tasks) ) {
					return;
				}
				
				$.each($tasks, function(){
					var title, odata = {}, num = 0;
					if (this.GovernanceState) {
						var state = this.GovernanceState.Value;
						var sla = this.SLA;
						num = this.Total;
						title = state.Name;
						odata = {
							ObjId: state.ObjId,
							Template: state.Template,
							Pending: sla.Pending,
							Critical: sla.Critical,
							Delayed: sla.Delayed, 
							Total: num
						};
					}
					else {
						$.each(this, function() {
							if (String(this.behaviour).match(/sleeping|idle/i)) {
								return;
							}
							title = this.Name;
							odata = this;
						});
					}
					
					data.result.push({
						title: title,
						data: odata,
						lazy: num ? true : false,
						folder: num ? true : false
					})
				});
			},
			collapse: function() {
				var data = {Content: $tree.fancytree("getTree").toDict(), Total: $tree.data("counter")};
				$.browserCache("set", "Tasks" + $QisSession.RepId, window.JSON.stringify(data));
			},
			expand: function() {
				var data = {Content: $tree.fancytree("getTree").toDict(), Total: $tree.data("counter")};
				$.browserCache("set", "Tasks" + $QisSession.RepId, window.JSON.stringify(data));
			}
		});

		$tree.ajaxLoaderText(false);
		callback.call($tree, clbParm);
	}
		
	if (cache) {
		var $data = $.browserCache("get", "Tasks" + $QisSession.RepId);
		if ($data) {
			$data = $.parseJSON($data);
			if ($data.Content.length) {
				buildTreeView($data.Content);
				$tree.data("counter", $data.Total).data("domevent", false);
				return $tree;
			}
		}
	}
  
	$.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		url: wsurl,
		data: window.JSON.stringify({	
			objectAttributes: [
				{Name:"ConfigurationFilter", Format:"PlainText", Value: ConfIdList}
			]})
	})
	.done(function (data) {
		if (!isValidJsonData.call(null, data, "GetGovernanceTasksTree")) {
			data = "";
		}
		
		if (!data.length) {
			return $tree.empty();
		}
		
		var $data = $.parseJSON(data);		
		if (!$data.Total) {
			$tree.html(emptytxt);
			callback.call($tree, clbParm);
			return;
		}
		
		$tree.data("counter", $data.Total).data("domevent", false);
		
		var $ul = $("<ul/>")
			.appendTo($tree);
			
		$.each($data.GetGovernanceTasks, function(){
			var gwe = this.GovernanceWorkFlow.Value;
			var sla = this.SLA;
			var num = this.Total;
			$ul.append($("<li/>")
				.addClass("folder lazy")
				.data("json", {
					ObjId: gwe.ObjId,
					Template: gwe.Template,
					Pending: sla.Pending,
					Critical: sla.Critical,
					Total: num,
					title: $.LL(gwe.Name)
				})
				.text(gwe.Name));
		});
		
		buildTreeView();
		var data = {Content: $tree.fancytree("getTree").toDict(), Total: $tree.data("counter")};
		$.browserCache("set", "Tasks" + $QisSession.RepId, window.JSON.stringify(data));
	})
	.fail(function(jqxhr, textStatus, error){
		notifyRequestError(jqxhr, textStatus, error, wsurl, null, null);
		$tree.html(emptytxt);
		callback.call($tree, clbParm);
		return;
	})
	
	return $tree;
}

RepositoryObject.prototype.GovernanceStatus = function () { 
	return this.status.apply(this, arguments);
}

RepositoryObject.prototype.GovernanceActions = function () {
	var args = arguments;
    var $obj = this;
	
	if (!$obj.TemplateDefinition) {
		return $obj.TemplateDefinitions(function(){
			$obj.GovernanceActions.apply($obj, args);
		});
	}
	
    var fnc = args[0] || false;
	var $cnt = args[1] || false;
	var forc = true;
		
	if (typeof args[2] != "undefined") {
		forc = args[2];
	}
 
	var $hdr = $cnt ? $cnt : $("#contentHeading");

    var $list = $cnt ? 
		$cnt.find("span.governanceactions") : 
		$("span.governanceactions");
	
    if ($list.length) {
		if ($list.isLoading() && !forc) {
			return $list;
		}
        $list.empty();
    }
    else {
        $list = $("<span />")
            .addClass("governanceactions buttons")
            .css({
                "float": "right",
                "font-weight": "normal",
                "font-size": "15px",
                "margin-top": "-1px"
            })
			.appendTo($hdr);
    }

    $list
		.ajaxLoaderText()
		.find(".ajaxloader")
			.css("float", "left")
			.css("font-size", $cnt ? "12px" : "14px");
	
	var width = $hdr.width() 
		- $hdr.find("h3").children("span").first().width()
		- $hdr.find("rateit").width();
		
	var nbtns = parseInt(width / 180);
	
	var loading = 2;
	
	var actionOnClick = function() {
		$list.empty()
			.ajaxLoaderText(true, $QepSettings.Definitions.Governance.GovernanceActionOnClick);
	};
	
	var handleGovernanceAction = function() {
		var func = arguments[0];
		var item = arguments[1];
		var $lst = arguments[2];

		if (item.RequiresAuthentication || item.RequiresFullAuthentication) {
			initQisAuthentication.call(item,  function(){
				$obj.GweGenericFunction(func, item, $lst, item.UseAuthenticationLogin ? this : null);
			}, function(){
				$obj.GovernanceActions();
			});
		}
		else {
			$obj.GweGenericFunction(func, item, $lst);
		}
	};
	
	var $parms = { objectAttributes: [
		{Name:"sys_ObjectId", Format:"PlainText", Value:$obj.Data.ObjId},
		{Name:"Type", Format:"PlainText", Value:"Transition"}]
	};	
	
	$.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		url: initQisCsharpMethod.call($obj, "GWE_ServiceHandlers.GetGovernanceActions"),
		data: window.JSON.stringify($parms)
	})
	.done(function (data) {
		if (!isValidJsonData.call($obj, data, "GetGovernanceActions")) {
			data = "";
		}

		if (!(--loading)) {
			$list.ajaxLoaderText(false);
		}
		
		var $data = data.length ? $.parseJSON(data) : {GovernanceFeatures:{}};
		var $tlist = $("<span/>").appendTo($list);

		$.each($data.GovernanceTransitions || {}, function (key, transition) {
			if (!$tlist.children(".actionbuttons").length) {
				$tlist.addDropdownButton($QepSettings.Definitions.Governance.GovernanceActionText, "status.png", "actionbuttons");
				$tlist.addButton("", "ui-icon-help").on($.browser.click, function(e){
					e.preventDefault();
					$obj.GovernanceWorkFlowView();
				}).css("width", "1.7em");
			}
			$tlist.children(".actionbuttons:first").addDropdownButtonItem(transition.Title, transition.Icon, function () { 
				handleGovernanceAction(fnc, transition, $tlist);
			}, $tlist)
			.on($.browser.click, function(){
				actionOnClick.call();
			});
		});

		if (typeof fnc == "function") {
			fnc();
		}
	})
	.fail(function(){
		$.browser.error("Failed getting governance transitions")
		$list.ajaxLoaderText(false);
	});
	
	$parms = { objectAttributes: [
		{Name:"sys_ObjectId", Format:"PlainText", Value:$obj.Data.ObjId},
		{Name:"Type", Format:"PlainText", Value:"Features"}]
	}
	
	var $defs = $QepSettings[$obj.TemplateDefinition.SettingsFile];
	if ($defs) {
		if ($defs.Governance.CustomActions) {
			$parms.objectAttributes.push({
				Name:"TemplateDefinition", Format:"PlainText", Value:$defs.Information.ObjId
			});
		}
	}
	
    $.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		url: initQisCsharpMethod.call($obj, "GWE_ServiceHandlers.GetGovernanceActions"),
		data: window.JSON.stringify($parms)
	})
	.done(function (data) {
		if (!isValidJsonData.call($obj, data, "GetGovernanceActions")) {
			data = "";
		}
		
		if (!(--loading)) {
			$list.ajaxLoaderText(false);
		}
		else {
			$list.find(".ajaxloader")
				.css("float", "right");
		}
		
		var $data = data.length ? $.parseJSON(data) : {GovernanceFeatures:{}};	

		var $flist = $("<span/>").prependTo($list);
		var compact = $.cookie("_ResponsiveLeft_") || $.keyCount($data.GovernanceFeatures) > (nbtns-1)
	   	$.each($data.GovernanceFeatures || {}, function (key, feature) {
			if (!isValidConfiguration(this.ConfigurationFilter)
			|| !isValidRole(this.RoleFilter)
			|| !$obj.ValidCondition(this.ConditionFilter)) {
				return true;
			}
			if (String(feature.CSMethod).match(/webmodeler/i)) {
				loadGraphicalXmlData($obj);
			}
			if (compact) {
				if (!$flist.children(".featurebuttons").length) {
					$flist.addDropdownButton($QepSettings.Definitions.Governance.GovernanceFeatureText, "comment.png", "featurebuttons");
				}
				if(feature.FeatureType == "Custom")
				{	
						//$.parse(feature.JavaScriptFeature)[0];
					 eval(feature.JavaScriptFeature.replace(/&quot;/g,'"').replace(/&amp;/g,"&").replace(/&lt;/g,"<").replace(/&gt;/g,">"));
				}
				else 
				{
					$flist.children(".featurebuttons:first").addDropdownButtonItem(feature.Title, feature.Icon, function () {
						handleGovernanceAction(fnc, feature, $flist);
					}, $flist)
					.on($.browser.click, function(){
						actionOnClick.call();
					});
				}
				
			}
			else {
				if(feature.FeatureType == "Custom") 
				{
						//$.parse(feature.JavaScriptFeature)[0];
					 eval(feature.JavaScriptFeature.replace(/&quot;/g,'"').replace(/&amp;/g,"&").replace(/&lt;/g,"<").replace(/&gt;/g,">"));
				}
				else 
				{
					$flist.addButton(feature.Title, feature.Icon, null, function () {
						handleGovernanceAction(fnc, feature, $flist);
					})
					.on($.browser.click, function(){
						actionOnClick.call();
					});
				}
			}
		});

		if (typeof fnc == "function") {
			fnc();
		}
	})
	.fail(function(){
		$.browser.error("Failed getting governance features")
		$list.ajaxLoaderText(false);
	});

    return $list;
}

$.ControlDialog = function() {
	var checklistVal = arguments[0];
	
	var $cnt  = $("<form id=\"checkresult\" style=\"overflow:auto;border:1px solid #3d3d3d;padding:0px 0px 0px 5px\" />")
		.append($("<h1 style=\"text-align:center\"/>").html(checklistVal.Value.Name))
			.width(500)
			.height(800);								
	$.fancybox($cnt,{modal:true,beforeClose:function() { $$.GovernanceActions(); }});
	

	var $cnt1 = $("<span id=\"buttons\" />")
		.insertAfter(".fancybox-inner")	
		var pageRefresh = function() { 
				$.browserCache("clear");	
				$.pageCache("clear");
				$.initPageContent(null, true);			
		};
		var $btn = $cnt1
				.append("<button class=\"btnSave\">" + $.LL("Save") + "</button>")
				.append("<button class=\"btnApprove\">" + $.LL("Approve") + "</button>")
				.append("<button class=\"btnCancel\">" + $.LL("Cancel") + "</button>")
				.append("<button class=\"btnDelete\">" + $.LL("Delete") + "</button>")
				.css({
					"text-align": "right",
					"margin-top": "2em"
				})
				.find("button").css({
					"line-height": "2em",
					"font-size": "0.9em",
					"margin-left": "4px"
				}).end()
				.find(".btnApprove").hide().end()
				.find(".btnDelete").hide().end();

				var $listOfCheckItems = [];
				var ObjStatus = "";
			
				var checklistObj = new RepositoryObject(checklistVal.Value.ObjId);
				checklistObj.Info("CheckItems", function(o) {
				var radioval;
				if(o.CheckItems!=null && o.CheckItems != "") {				
					$.each(o.CheckItems.Value, function() {
						var checkitem = this;
						var checkitemObj = new RepositoryObject(checkitem.ObjId);
						var $cnt2 = $("<div style=\"padding:5px 0px 5px 0px\"/>").appendTo($cnt).ajaxLoaderText();
						checkitemObj.init(function(q) {
						
								checkitemObjParsed = checkitemObj;
								
								var objIdText = checkitemObj.attr("ObjId").text();
								var typeText = checkitemObj.attr("Type").text();
								var nameText = checkitemObj.attr("Name").text();
								var optParaText = checkitemObj.attr("OptionalTypeParameters").text();
								
								$cnt2.append("<hr style=\"width:98%\">");						
									$cnt2.append("<b>"+nameText+"</b><br/>");
									var desc ="";
									var temp = nameText.replace(/\./g,' ');
									var idName = temp.replace(/\s+/g, '');
									
									var eachPara = [];
									if(optParaText != "") {
										var OptTypePara = checkitemObj.attr("OptionalTypeParameters").text();
										var replaced = OptTypePara.replace(/\n|\r|<p>|<\/p>/g, "¤");
										eachPara = replaced.split("¤");
										eachPara.remove("");
									}
									//Description Field for each CheckItem				
									if(checkitemObj.attr("ShortDescription").text() != "")
									{
										desc = 	checkitemObj.attr("ShortDescription").text();	
										$cnt2.append(desc + "</br>");
									}
										
									if(typeText != "") 
									{
										//TextField
										if(typeText == "Text") {
											$cnt2.append($("<textarea type=\"textarea\" name=\""+objIdText+"\" style=\"height:100px;width:400px;\"/>"));		
											$listOfCheckItems.push({id: objIdText, name: nameText, type: typeText, description:desc});
										}
										
										//CheckBox
										if(typeText == "Checkbox") {
											if(eachPara.toString() != "") {
												var maxArray = eachPara[eachPara.length-1].split(":");
													var max = maxArray[1].trim();

													var minArray = eachPara[eachPara.length-2].split(":");
													var min = minArray[1].trim();
													
												for(var i = 0; i < eachPara.length-2; i++) {
												$cnt2.append($("<input type=\"checkbox\" id=\""+eachPara[i]+"\" class=\""+objIdText+"\" name=\""+objIdText+"\" max=\""+max+"\"  min=\""+min+"\" value=\""+eachPara[i]+"\">"),eachPara[i]);
												
												if(i != eachPara.length-3) {
														 $cnt2.append("</br>");
												}
														 $listOfCheckItems.push({id: objIdText, name: eachPara[i], type: typeText, description:desc});
												}
												
											}
											$("."+objIdText+"").on('change', function(e){
													$("."+objIdText+"").html("");
											
													if((parseInt(min) <= $("."+objIdText+":checkbox:checked").length) && ($("."+objIdText+":checkbox:checked").length <= parseInt(max))) {
														$("."+objIdText+"").html("");	
													}
													else if($("."+objIdText+":checkbox:checked").length > parseInt(max)) {
														$("."+objIdText+"").html("");
														$("."+objIdText+"").html("</br>Info: Please choose maximum "+max+" boxes").css({"color":"red"});	
													}
													else if(!(parseInt(min) < $("."+objIdText+":checkbox:checked").length)) {
														$("."+objIdText+"").html("");
														$("."+objIdText+"").html("</br>Info: Please choose minimum "+min+" boxes").css({"color":"red"});	
													}
													else 
													{
														$("."+objIdText+"").html("");
													}
											});

												$cnt2.append("<span class=\""+objIdText+"\"><span>");
										}
										
										//Radio button
										if(typeText == "Radio") {
											if(eachPara.toString() != "") {
												for(var i = 0; i < eachPara.length; i++) {
													$cnt2.append($("<input type=\"radio\" name=\""+objIdText+"\" value=\""+eachPara[i]+"\">"),eachPara[i]);
													
													if(i != eachPara.length-1) {
														 $cnt2.append("</br>");
													}
													$listOfCheckItems.push({id: objIdText, name: eachPara[i], type: typeText, description:desc});
												
												}												
											}
										}
										//Bool radio button
										if(typeText == "Bool") {
										if(eachPara.toString() != "") 
										{
											$cnt2.append($("<input type=\"radio\" name=\""+objIdText+"\" value=\""+eachPara[0]+"\">"),eachPara[0]+"</br>");
											$cnt2.append($("<input type=\"radio\" name=\""+objIdText+"\" value=\""+eachPara[1]+"\">"),eachPara[1]);
											$listOfCheckItems.push({id: objIdText, name: eachPara[0], type: typeText, description:desc});
											$listOfCheckItems.push({id: objIdText, name: eachPara[1], type: typeText, description:desc});
										}
										else 
										{
											$cnt2.append($("<input type=\"radio\" id=\"True\" name=\""+objIdText+"\" value=\"True\"> "),"True </br>");
											$cnt2.append($("<input type=\"radio\" id=\"False\" name=\""+objIdText+"\" value=\"False\">"),"False");
											$listOfCheckItems.push({id: objIdText, name: "True", type: typeText, description:desc});
											$listOfCheckItems.push({id: objIdText, name: "False", type: typeText, description:desc});
										}
											
											
										}
										
										//Number
										if(typeText == "Number") {
												if(eachPara.length != 0) {
											
													$cnt2
													.append("Number between "+eachPara[0]+" and "+eachPara[1]+"<br/>")
													.append($("<input type=\"number\" id=\""+idName+"\" name=\""+objIdText+"\" min=\""+eachPara[0]+"\" max=\""+eachPara[1]+"\" style=\"width:400px\">").change(
														function() {															
															if(parseInt(eachPara[0]) <= $(this).val() &&  $(this).val() <= parseInt(eachPara[1])) {
																$("."+idName+"").html("");
																$("#WithNonConformance").prop('checked', false);
															}
															else 
															{
																$("."+idName+"").html("");
																$("."+idName+"").html("</br>Info: The number does not match the allowed range - create a non conformance when finished").css({"color":"red"});
																$("#WithNonConformance").prop('checked', true);
															}
														}));
														$listOfCheckItems.push({id: objIdText, name: nameText, type: typeText, description:desc});
													$cnt2.append("<span class=\""+idName+"\"><span>");
												}
												else 
												{
													$cnt2.append($("<input type=\"number\" id=\""+idName+"\" name=\""+objIdText+ "\" style=\"width:400px\">"));
													$listOfCheckItems.push({id: objIdText, name: nameText, type: typeText, description:desc});
												}
											
										}
										//Date
										if(typeText == "Date") {
										
										$cnt2.append("<input type=\"text\" id=\""+objIdText+"\" name=\""+objIdText+"\" value=\"Choose date\" />");
										$(function() {

											$( "#"+objIdText+"" ).datepicker();

										  });
	
											$listOfCheckItems.push({id: objIdText, name: nameText, type: typeText, description:desc});

										}
									}
								$cnt2.append("</br>");

							$cnt2.ajaxLoaderText(false);
						});
					});
			
					$.ajax({
						type: 'POST',
						dataType: 'JSON',
						contentType : 'application/json',
						url: initQisCsharpMethod("Asset_ServiceHandlers.GetCheckResult"),
						data: window.JSON.stringify({
							objectAttributes:[
								{Name:"ChecklistID", Format:"PlainText", Value: checklistVal.Value.ObjId},
								{Name:"Concerns", Format:"PlainText", Value: $$.Data.ObjId}
							]
						})
					}).done(function(data) {
						if(data) {
					
						var $resultList = [];
					
						var loadExistingData = function(){
						
							var parsedXML = $.parseXML(data);
							var checklist = $(data).find('checklist');
						
							$(data).find('checkitem').each(function(){
								var oid = $(this).find('oid').text();
								var val = $(this).find('result').text();
									$resultList.push({id: oid, value:val});
								});					
						};						
							$.async(function(){
								if ($cnt.isLoading()){
									return;
								}
								$.async(false, this);
								loadExistingData();
								
								var textareas = $cnt.find('textarea');
								textareas.each(function() {
								var inputItem = $(this)[0];
								var nameId = inputItem.name;
									var inputVal = inputItem.value;
									for(var i = 0; i < $resultList.length;i++) {
										var resultItem = $resultList[i];
										if(resultItem.id == nameId) {
											this.value = resultItem.value;
										}
									}								
								});
								
								var checkitem = $cnt.find('input');	
								
								$cnt.find('input').each(function()  { 								
									var inputItem = $(this)[0];
									var nameId = inputItem.name;
									var inputVal = inputItem.value;
									for(var i = 0; i < $resultList.length;i++) {
										var resultItem = $resultList[i];
										if(resultItem.id == nameId) {
											switch(inputItem.type) {
												
												case "text":												
														inputItem.defaultValue = resultItem.value;												
													break;
												 case "radio":
													 if(inputItem.defaultValue == resultItem.value) {
														this.defaultChecked = true;
													 }
													break;
											case "number":
													inputItem.defaultValue = resultItem.value;
												
													 if(parseInt(this.min) <= $(this).val() &&  $(this).val() <= parseInt(this.max)) {
														$("."+this.id+"").html("");																	
													}
													else {
														$("."+this.id+"").html("");
														$("."+this.id+"").html("</br>Info: The number does not match the desired range - create a non conformance when finished").css({"color":"red"});;
														$("#WithNonConformance").prop('checked', true);
													}
													break;
											case "checkbox":
												if(inputItem.defaultValue == resultItem.value) {
														this.defaultChecked = true;
														var max = this.max;
														var min = this.min;
														
														if((parseInt(min) <= $("."+nameId+":checkbox:checked").length) && ($("."+nameId+":checkbox:checked").length <= parseInt(max))) {
															$("."+nameId+"").html("");	
														}
														else if($("."+nameId+":checkbox:checked").length > parseInt(max)) {
															$("."+nameId+"").html("");
															$("."+nameId+"").html("</br>Info: Please choose maximum "+max+" boxes").css({"color":"red"});	
														}
														else if(!(parseInt(min) < $("."+nameId+":checkbox:checked").length)) {
															$("."+nameId+"").html("");
															$("."+nameId+"").html("</br>Info: Please choose minimum "+min+" boxes").css({"color":"red"});	
														}
														else 
														{
															$("."+nameId+"").html("");
														}
													 }
													 break;
											default:
												break;
											}
										}
								}
								});
								
								$btn.find(".btnDelete").show()
									.click(
										function() 
										{
											$.ajax({
												type: 'POST',
												dataType: 'JSON',
												contentType : 'application/json',
												url: initQisCsharpMethod("Asset_ServiceHandlers.DeleteCheckResult"),
												data: window.JSON.stringify({
													objectAttributes:[
														{Name:"ChecklistId", Format:"PlainText", Value: checklistVal.Value.ObjId},
														{Name:"AssetID", Format:"PlainText", Value: $$.Data.ObjId},
													]
												})
											});
											
												$.fancybox.close();
												pageRefresh();
												
												
										}
									);
							}, 100, true);
						}
					}).fail(function() {
						$.browser.error("FAIL");
					});
				}
				
			});  

		$.ajax({
			type: 'POST',
			dataType: 'JSON',
			contentType : 'application/json',
			url: initQisCsharpMethod("Asset_ServiceHandlers.GetLockStatus"),
			data: window.JSON.stringify({
				objectAttributes:[
					{Name:"ChecklistId", Format:"PlainText", Value: checklistVal.Value.ObjId}
				]
			})
			}).done(function(data) {	
				 if(data != "Locked") {
					$btn.find(".btnApprove").show()
					.click(
						function() 
						{
							var $cntXML = "<xml><checklist><oid>"+checklistVal.Value.ObjId+"</oid>";
							var resultlist2 = JSON.stringify($("#checkresult").serializeArray());			 
							var list = $.parseJSON(resultlist2);
							for(var listitem = 0; listitem < list.length; listitem++)
							{
								var objidList = list[listitem].name;
								var inputValue = list[listitem].value;
								for(var i = 0; i < $listOfCheckItems.length;i++) 
								{
					
									if(objidList == $listOfCheckItems[i].id && inputValue != "") {
									var repObj = new RepositoryObject(objidList);
								
									var objName = repObj.Data.Name;
										if($listOfCheckItems[i].type == "Checkbox" || $listOfCheckItems[i].type == "Radio" || $listOfCheckItems[i].type == "Bool") {
											if($listOfCheckItems[i].name == inputValue) {
											$cntXML = $cntXML + "<checkitem><checklistname>"+objName+"</checklistname> <oid>"+objidList+"</oid><name>"+$listOfCheckItems[i].name+"</name>"+
											"<type>"+$listOfCheckItems[i].type+"</type>"+
											"<result>"+inputValue+"</result>"+
											"</checkitem>";
										}
										}
										else {
											if(inputValue != "Choose date") {
												$cntXML = $cntXML + "<checkitem><checklistname>"+objName+"</checklistname>  <oid>"+objidList+"</oid><name>"+$listOfCheckItems[i].name+"</name>"+
											"<type>"+$listOfCheckItems[i].type+"</type>"+
											"<result>"+inputValue+"</result>"+
											"</checkitem>";
											}
										}
									}
								}
							}
							$cntXML = $cntXML + "</checklist></xml>";
			
							var checked = $('#WithNonConformance').is(":checked");
							$.ajax({
								type: 'POST',
								dataType: 'JSON',
								contentType : 'application/json',
								url: initQisCsharpMethod("Asset_ServiceHandlers.CreateCheckResult"),
								data: window.JSON.stringify({
									objectAttributes:[
										{Name:"ChecklistID", Format:"PlainText", Value: checklistVal.Value.ObjId},
										{Name:"Value", Format:"PlainText", Value: $cntXML},
										{Name:"AssetID", Format:"PlainText", Value: $$.Data.ObjId},
										{Name:"Submit", Format:"PlainText", Value: "Submit"}
									]
								})
							}).done(function() {
								if(checked) {
								$.ajax({
										type: 'POST',
										dataType: 'JSON',
										contentType : 'application/json',
										url: initQisCsharpMethod("Asset_ServiceHandlers.CreateNonConformance"),
										data: window.JSON.stringify({
											objectAttributes:[
												{Name:"ChecklistID", Format:"PlainText", Value: checklistVal.Value.ObjId},
												{Name:"AssetID", Format:"PlainText", Value: $$.Data.ObjId},
												{Name:"UserID", Format:"PlainText", Value: $QisSession.UserInfo.ObjId}
											]
										})
									});	
								}									
							});
								
							$.fancybox.close();
							pageRefresh();
						}
					);
						$cnt1.append($("<input type=\"checkbox\" id=\"WithNonConformance\" name=\"WithNonConformance\" value=\"WithNonConformancee\">"),$.LL("Create Non Conformance with approve"));
							
							if(data == "GrabLock") {
								$cnt1.append("<br/>N.B. You have the Asset open in another session <br/> - approving or saving the control will result in loss of unsaved changes.");
							}
						}
						else {
							$cnt1.append(" N.B. Cannot approve check (Asset is opened in another session)").css({"color":"red"});
						}	
				});
				
				$btn.find(".btnSave").click(
					function() 
					{
			
						var $cntXML = "<xml><checklist><oid>"+checklistVal.Value.ObjId+"</oid>";
							var resultlist2 = JSON.stringify($("#checkresult").serializeArray());			 
							var list = $.parseJSON(resultlist2);
							for(var listitem = 0; listitem < list.length; listitem++)
							{
								var objidList = list[listitem].name;
								var inputValue = list[listitem].value;
								for(var i = 0; i < $listOfCheckItems.length;i++) 
								{
									if(objidList == $listOfCheckItems[i].id && inputValue != "") {
									var repObj = new RepositoryObject(objidList);
								
									var objName = repObj.Data.Name;
										if($listOfCheckItems[i].type == "Checkbox" || $listOfCheckItems[i].type == "Radio" || $listOfCheckItems[i].type == "Bool") {
											if($listOfCheckItems[i].name == inputValue) {
											$cntXML = $cntXML + "<checkitem><checklistname>"+objName+"</checklistname> <oid>"+objidList+"</oid><name>"+$listOfCheckItems[i].name+"</name>"+
											"<type>"+$listOfCheckItems[i].type+"</type>"+
											"<result>"+inputValue+"</result>"+
											"</checkitem>";
										}
										}
										else {
											if(inputValue != "thevalue") {
												$cntXML = $cntXML + "<checkitem><checklistname>"+objName+"</checklistname>  <oid>"+objidList+"</oid><name>"+$listOfCheckItems[i].name+"</name>"+
											"<type>"+$listOfCheckItems[i].type+"</type>"+
											"<result>"+inputValue+"</result>"+
											"</checkitem>";
											}
										}
									}
								}
							}
							$cntXML = $cntXML + "</checklist></xml>";
					
						$.ajax({
							type: 'POST',
							dataType: 'JSON',
							contentType : 'application/json',
							url: initQisCsharpMethod("Asset_ServiceHandlers.CreateCheckResult"),
							data: window.JSON.stringify({
								objectAttributes:[
									{Name:"ChecklistID", Format:"PlainText", Value: checklistVal.Value.ObjId},
									{Name:"Value", Format:"PlainText", Value: $cntXML},
									{Name:"Concerns", Format:"PlainText", Value: $$.Data.ObjId},
									{Name:"Save", Format:"PlainText", Value: "save"}
								]
							})
						});
						
						$.fancybox.close();
						pageRefresh();
					}
				);
				
				$btn.find(".btnCancel")
				.click(
					function() 
					{
						$.fancybox.close();
					});
				
				$btn.appendTo($cnt1).find("button").button();
	
}
RepositoryObject.prototype.RatingScore = function() {
	var $obj = this;
	var $cnt = arguments[0];
	var $rating = $cnt ? $cnt.find(".ratingcontrol") : $("div.ratingcontrol");
	
	if ( !$rating.length ) {
        $rating = $("<div />")
            .addClass("ratingcontrol rateit")
            .css({
                "margin": "1px 5px 0 5px",
				"width": "80px",
				"float": "left",
				"line-height": "24px"
            })
			.hide();
				
			$.getScript("js/rateit/jquery.rateit.min.js").done(function(){
				$rating.rateit().rateit('readonly',true);
			});
	}
	
	$rating.hide();
	
	$.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		url: initQisCsharpMethod.call($obj, "GWE_ServiceHandlers.GetRating"),
		data: window.JSON.stringify({
			objectAttributes: [
				{Name:"sys_ObjectId",Format:"PlainText",Value:$obj.Data.ObjId},
				{Name:"sys_RevisionId",Format:"PlainText",Value:$obj.Data.RevisionId}
			]
		})
	})
	.done(function(data){
		if (!isValidJsonData.call($obj, data, "GetRating")) {
			data = "{}";
		}
		
		var $data = data.length ? $.parseJSON(data) : {};
		$obj.Data.Rating = $data;
		$.each($obj.Data.Rating, function(){
			 $rating
				.show()
				.rateit('value', this.AverageRatingScore)
				.rateit('ispreset',true);
			return false;
		})
	})
	.fail(function(){
		$.browser.error("Failed getting rating score")
		$rating.remove();
	})
	
	return $rating;
}

RepositoryObject.prototype.GweGenericFunction = function () {
	var $obj = this;
	var args = arguments;
	
	var fnc = args[0] || null;
	var feature = args[1] || null;
	var $cnt = args[2] || $();
	var forceTransition = args[3] || false;
	var customSid = args[4] || null;
	var comment = null;
	
	if (typeof args[5] != "undefined") {
		comment = args[5];
	}
	
	var configuration = getConfigurationInfo(feature.ExecutionConfiguration, {SkipDisabled: false});
	var customization = feature.Customization || feature.CSMethod;
	var state = feature.Title;
	var requirement = feature.CommentRequirement || "Skip";
	
	var setUserSession = $.noop;

	if (feature.FeatureTemplate=="GovernanceTransition"){
		feature.Template="Icon";
		if (feature.FeatureType!="WebForm") {
			customization += state;			
			if (!requirement.match(/skip/i) && comment===null) {
				return $.CommentDialog.call(feature, function( val ){
					if (val === null) {
						return $obj.GovernanceActions();
					}
					args["length"] = 6;
					args[5] = val;
					return $obj.GweGenericFunction.apply($obj, args);
				}, requirement.match(/optional/i));
			}
		}
	}
	
	if (customSid) {
		setUserSession = function(sid) {
			if (typeof sid != "undefined") {
				$.cookie("OldUserSid", $.cookie("UserSid"), {path:'/'});
				$.cookie("UserSid", sid, {path:'/'});
			}
			else if ($.cookie("OldUserSid")){
				$.cookie("UserSid", $.cookie("OldUserSid"), {path:'/'});
				$.removeCookie("OldUserSid", {path:'/'});
			}
		}
		
		setUserSession(customSid);
	}
    
	if (customization.match(/webmodeler/i)) {
		return $obj.DiagramEditor();
	}
	
	if (feature.FeatureType.match(/webform/i)) {
		$obj.ObjectDialog(customization,
			feature.Template,
			feature.Link,
			feature.Width,
			feature.Height,
			null,
			null,
			configuration.Id,
			function() {
				setUserSession.call();
				$.browserCache("del", "Tasks" + $QisSession.RepId);	
				$.browserCache("del", "Subs" + $QisSession.ConfId);	
				$obj.init(function(){
					$.initPageContent(null, true);
				}, null, null, true);
			}, 
			feature.Title
		);
	}
	else if (feature.FeatureType.match(/dropdown|custom/i)) {
		fnc.call(feature);
		return;
	}
	else {
		var governanceTransitionAjax = function() {
			if (!arguments.length) {
				reurn;
			}
			
			var webServiceMethod = arguments[0];
			
			return $.ajax({
				type: 'POST',
				dataType: 'JSON',
				contentType : 'application/json',
				url: initQisCsharpMethod.call($obj, webServiceMethod),
				data: window.JSON.stringify({
					objectAttributes:[
						{Name: "sys_ObjectId", Format: "PlainText", Value: $obj.Data.ObjId},
						{Name: "sys_RevisionId", Format:"PlainText", Value: $obj.Data.RevisionId},
						{Name: "state", Format: "PlainText", Value: state},
						{Name: "comment", Format: "PlainText",Value: comment},
						{Name: "forceTransition", Format: "PlainText", Value: forceTransition}
					]
				})
			});
		};

		if (feature.FeatureTemplate == "GovernanceStateFeature") {
			if (customization == "GWE_ServiceHandlers.RateObject") {
				$.RatingDialog.call($obj, fnc, feature, function(ret, fncDyn, featureNew) { 
					if (ret) { 
						ret.objectAttributes.push({Name: "sys_ObjectId", Format: "PlainText", Value: $obj.Data.ObjId});
						$.ajax({
							type: 'POST',
							dataType: 'JSON',
							contentType : 'application/json',
							url: initQisCsharpMethod.call($obj, customization),
							data: window.JSON.stringify(ret)
						})
						.always(function (data) {
							if (data.status != 200){
								return;
							}
							setUserSession.call();
							$obj.RatingScore();
							$obj.GovernanceActions();
						});
					}
				}); 
				return;
			}
			else if (customization) {
				governanceTransitionAjax(customization)
					.always(function (data) {
						if (data.status != 200) {
							return;
						}
						setUserSession.call();
						if (customization.match(/subscribe/i)) {
							$.jnotify({
								message: $.LL("Subscriptions updated")
							});
							$.browserCache("del", "Subscriptions" + $QisSession.ConfId);
							$obj.UserSubscriptions(false);
						}
						if (customization.match(/acknowledge/i)) {
							$.jnotify({
								message: $.LL("Acknowledgement updated")
							});
							$.browserCache("del", "Tasks" + $QisSession.ConfId);
							$obj.GovernanceTasks(false);
						}
						$obj.GovernanceActions();
					}); 
				return;
			}
		}
				
		governanceTransitionAjax("GWE_ServiceHandlers.DoGovernanceTransition")
			.always(function (data) {
				if (!isValidJsonData.call($obj, data, "DoGovernanceTransition")) {
					data = "";
				}
				
				var $data = data.length ? $.parseJSON(data) : { GovernanceTransitionResult:{ Result: "", Messages: {} }};
				var result = $data.GovernanceTransitionResult.Result || "";
				if (result.toLowerCase() !== "ok") {
					setUserSession.call();
					return $.WarningDialog($data.GovernanceTransitionResult, function(val) {
						if (val === false) {
							return $obj.GovernanceActions();
						}
						$obj.GweGenericFunction(fnc, feature, null, true, customSid, comment);
					});
				}
				$.jnotify({
					title: feature.Link,
					message: $.LL("Transition successfully completed"),
					lifespan: 2000
				});
				setUserSession.call();
				$.browserCache("del", "Subscriptions" + $QisSession.ConfId);	
				$.browserCache("del", "Tasks" + $QisSession.RepId);	
				$obj.cache(false);
								
				var cid = getConfigurationInfo(configuration.Id).Id;
				if ($QisSession.ConfId == cid) {
				$obj.status( function(){
					$obj.init(
						function(){ $.initPageContent(null, true); },
						null,
						function(){ return $.LL("{Name} has changed status: <b>{GovernanceStatus}</b>" + "<br/><br/>", $obj, true);},
						true
					);
				}, true);
				}
				else {
					var oid = (getUrlParams("oid") ? getUrlParams("oid") : $obj.Data.ObjId);
					var lcd = getUrlParams("lc");
					var href = $QisSession.HTMLLocation 
						+ (oid ? "?oid=" + oid : "")
						+ (lcd ? "&lc=" + lcd : "")
						+ (cid != $QisSession.BaseConfId ? "&cid=" + cid : "");
					
					$.async(function(){
						location.href = href;
					});
				}
			});
	}
}

$.WarningDialog = function() {
	if (!arguments.length) {
		return;
	}
	
	var $data = arguments[0]
	var callback = arguments[1] || $.noop;
	
	var $msgs = $data.Messages;
	var isErr = $data.Result.match(/error/i);	
	
	if (!$msgs && $data.Message) {
		$msgs = { 
			0: {
				Type: $data.Result,
				Message: $data.Message
			}
		};
	}
	
	var nmsgs = $.keyCount($msgs);
	
	var $cnt = $("<div/>").addClass("fancy-dialog warning-dialog")
		.width(500)
		.append($("<h3/>").css("margin-top", "0.5em")
			.html(nmsgs + " " + (nmsgs > 1 ? $.LL("Messages") : $.LL("Message"))));
	
	$.each($msgs, function() {
		var msgtext = String(this.Message || "").split("\n\n").unique().join("<br/>");
		var msgtype = String(this.Type || "").replace(/ok/i, $.LL("Passed"))	
				
		$cnt.append($("<div/>")
			.addClass("check-message")
			.addClass(msgtype.toLowerCase() + "-message")
			.append($("<h4/>")
				.append($("<span/>").addClass("message-icon"))
				.append($("<span/>").html($.LL(msgtype))))
			.append($("<div/>").html(msgtext)));
	});
		
	$btn = $("<div/>")
		.append("<button class=\"btnSubmit\">" + $.LL("Continue") + "</button>")
		.append("<button class=\"btnCancel\">" + (isErr ? $.LL("Close") : $.LL("Cancel")) + "</button>")
		.css({
			"text-align": "right",
			"margin-top": "2em"
		})
		.find("button").css({
			"line-height": "2em",
			"font-size": "0.9em",
			"margin-left": "4px"
		})
		.end()
		.appendTo($cnt);
		
	
	$btn.children(".btnSubmit").each(function(){
		$(this).button().on($.browser.click, function(){
			result = true;
			$.fancybox.close();
		});
	})
	.css({
		"display": isErr ? "none" : "inline-block"
	});

	$btn.children(".btnCancel").each(function(){
		$(this).button().on($.browser.click, function(){
			result = false;
			$.fancybox.close();
		});
	});
		
	$.fancybox($cnt, {
		modal: true,
		afterClose: function(){
			callback.call(null, result);
		}
	});
}

$.CommentDialog = function() {
	if (!arguments.length) {
		return;
	}
	
	var feature = this;
	
	var callback = arguments[0] || $.noop;
	var optional = arguments[1] || false;
	var ocomment = false;

	var $cnt = $("<div/>").addClass("fancy-dialog comment-dialog")
		.append(!feature.CommentText ? 
			$("<h3/>").html($.LL("Comment")) :
			$("<h4/>").html($.LL(feature.CommentText)))
		.append($("<textarea/>").attr("spellcheck", false)
			.css({"width": "388px", "height": "100px", "display": "block"}))
		.append($("<div/>").html($.LL("Invalid comment")).addClass("invalid-message"))
		.css({
			"width": "400px",
			"font-size": "9pt"
		})
		.find("h3").css({
			"margin-bottom": "1em", 
			"font-size": "1.3em",
			"padding-bottom": "0.4em",
			"border-bottom": "solid 1px #000"
		}).end()
		.find("h4").css({
			"margin-bottom": "1em", 
			"font-size": "1.1em"
		}).end()
		.find(".invalid-message").css({
			"position": "absolute",
			"bottom": "5px",
			"text-indent": "2px",
			"color": "red",
			"display": "none"
		}).end();
		
	$btn = $("<div/>")
		.append("<button class=\"btnSubmit\">&nbsp;" + $.LL("Submit") + "&nbsp;</button>")
		.append("<button class=\"btnSkip\">&nbsp;" + $.LL("Skip") + "&nbsp;</button>")
		.append("<button class=\"btnCancel\">" + $.LL("Cancel") + "</button>")
		.css({
			"text-align": "right",
			"margin-top": "2em"
		})
		.find("button").css({
			"line-height": "2em",
			"font-size": "0.9em",
			"margin-left": "4px"
		})
		.end()
		.appendTo($cnt);
		
	
	$btn.children(".btnSubmit").each(function(){
		$(this).button().on($.browser.click, function(){
			var $txt  = $cnt.find("textarea");
			ocomment = $txt.val();
			if (ocomment) {
				$.fancybox.close();
			}
			else {
				$cnt.find(".invalid-message").fadeIn();
				$.async(function(){
					$cnt.find(".invalid-message").fadeOut();
				}, 3000);
			}
		});
	});

	$btn.children(".btnSkip").each(function(){
		$(this).button().on($.browser.click, function(){
			ocomment = "";
			$.fancybox.close();
		});
	})
	.css({
		"display": optional ? "inline-block" : "none"
	});

	$btn.children(".btnCancel").each(function(){
		$(this).button().on($.browser.click, function(){
			ocomment = null;
			$.fancybox.close();
		});
	});
		
	$.fancybox($cnt, {
		modal: true,
		afterClose: function(){
			callback.call(null, ocomment);
		}
	});
}

$.RatingDialog = function() {
    if (!arguments.length) {
		return;
	}
	var $obj = this;
	
	var fnc = arguments[0];
	var ftr = arguments[1];
	var clb = arguments[2];
	var ret = null;
	
	var $cnt = $("<div/>").width(450)
		.append($("<h3/>").html("Rating").css({"margin": "0.2em 0", "font-size": "1.25em"}))
		.append("<div id=\"rateObject\" class=\"rateit\"/></div>")
		.append($("<div/>").html("<b>" + $.LL("Comment:") + "<b/>").css("margin","1em 0 0.2em 0"))
		.append($("<textarea/>").attr("spellcheck", false)
			.css({"width": "440px", "height": "100px", "display": "block"}));
		
	$btn = $("<div/>")
		.append("<button class=\"btnSubmit\">" + $.LL("Submit") + "</button>")
		.append("<button class=\"btnCancel\">" + $.LL("Cancel") + "</button>")
		.css({
			"text-align": "right",
			"margin-top": "1em"
		});
		
	$btn.children(".btnSubmit").each(function(){
		$(this).button().on($.browser.click, function(){
			ret = {
				objectAttributes: [
					{Name:"_RatingValue",Format:"PlainText",Value:$cnt.find("#rateObject").rateit("value")},
					{Name:"_RatingComment",Format:"PlainText",Value:$cnt.find("textarea").val()}
				]
			};
			$.fancybox.close();
		})
		.css({
			"line-height": "2em",
			"font-size": "0.9em",
			"margin-left": "4px"
		});
	})
	
	$btn.children(".btnCancel").each(function(){
		$(this).button().on($.browser.click, function(){
			$.fancybox.close();
			$obj.GovernanceActions();
		})
		.css({
			"line-height": "2em",
			"font-size": "0.9em",
			"margin-left": "4px"
		});
	});
					
	$.fancybox($cnt.append($btn), {
		modal: true,
		afterLoad: function() {
			if (typeof $.fn.rateit == "undefined") {
				$.getScript("js/rateit/jquery.rateit.min.js").done(function(){
					$cnt.find("#rateObject").rateit({step:1});
				});
			}
			else {
				$cnt.find("#rateObject").rateit({step:1});
			}
		},
		afterClose: function() {
			clb.call($obj, ret, fnc, ftr);
		}
	});
	
}

RepositoryObject.prototype.GovernanceWorkFlowView = function () {
	var $obj = this;
	var reload = arguments[0] || false;

	if (!reload) {
		$.overlay({loader: true, text: $.LL("Loading Governance Workflow...")});

		if (!$obj.InstantiationLevel.match("status").length) {
			$obj.status(function(){
				$obj.GovernanceWorkFlowView(true);
			});
			return;
		}
	}
	
	var $gwe = $obj.attr("GovernanceWorkFlow").find("a").first();	
	
	if (!$gwe.length) {
		$.overlay(false); 
		$.fancybox($("<h2>" + $.LL("Unable to find Governance Workflow...") + "</h2>"));
		return; 
	}
	
	var $wf = new RepositoryObject($gwe);
	$wf.init(function(){
		var $cnt = $("<div/>").append("<h2>" + $wf.Data.Name + "</h2>");
		var $desc = $wf.attr("ShortDescription")
			.css("max-width", 800)
			.appendTo($cnt);
		
		var initHeight = $desc.text().length*11*16/800 + 45;
		
		var $gws = $obj.attr("GovernanceCurrentState").find("a").first();
		
		loadGraphicalData($("<div/>"), $wf, true, null, false, function($o, url){
			var $graph = this;
			$cnt.append( $.initIFrameView({
				url: url,
				width: $graph.data("width"),
				height: $graph.data("height"),
				onLoad: function($ifrm){
					$ifrm.contents().find("a").each(function(){
						if ($(this).data("type") == "butpan") {
							$(this).detach().remove();	
						}
						else {
							$(this).disableHyperLink();
							if ($(this).data("objid") === $gws.data("objid")) {
								var $path = $(this).find("path:last");
								$path.each(function(){
									var sfillc = $(this).css("fill");
									var $state = $(this).css({
										"stroke": "rgba(0, 122, 60, 0.65)",
										"fill": "rgba(0, 122, 60, 0.35)"
									});
									var toggle = true;
									$.async(function(){
										if (!$state.length) {
											return $.async(false, this);
										}
										toggle = !toggle;
										$state.animate({
											"stroke-width": toggle ? 5 : 1
										}, 800)
									}, 1000, true);
									$.async(function(){
										$state.animate({
											"fill-opacity": 0.2
										}, 800, function(){
											$state.css("fill", sfillc).animate({
												"fill-opacity": 1
											}, 800);
										});
									}, 1000);
								});
							}
						}
					});
				}
			}));
			
			$cnt
				.height($graph.data("height") + initHeight)
				.width($graph.data("width") + 25);
			$.async(function(){
				$.overlay(false, true);
				$.fancybox($cnt);
			}, 1000)
		});
	}, null, false);
}
RepositoryObject.prototype.GovernanceHistory = function () {
	var $obj = this;
		
    var $opts = {
		container: $(),
        onLoad: function () { }
    }

    if (arguments.length) {
        $opts = $.extend($opts, arguments[0]);
        $opts.onLoad = arguments[0].onLoad;
    }

	$.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		url: initQisCsharpMethod.call($obj, "GWE_ServiceHandlers.GetGovernanceHistory"),
		data: window.JSON.stringify({
			objectAttributes: [
				{Name:"sys_ObjectId",Format:"PlainText",Value:$obj.Data.ObjId}
			]
		})
	})
	.done(function (data) {
		if (!isValidJsonData.call(null, data, "GetGovernanceHistory")) {
			data = "";
		}
		if(!data || data.length <= 0) {
			$opts.container
			.html($opts.emptytext)
			.css({
				"font-size": "1.3em",
				"margin": "0.65em 0.5em"
			});
			return;
		}
	   var $history = data.length ? $.parseJSON(data) : {};
	   $opts.onLoad($history);
	})
	.fail(function(){
		$.browser.error("Failed getting governance history")
		$opts.container
			.html($opts.emptytext)
			.css({
				"font-size": "1.3em",
				"margin": "0.65em 0.5em"
			});
	});
}

RepositoryObject.prototype.GenericDataTableQuery = function () {
	var $obj = this;
		
    var $opts = {
		container: $(),
		type: "full",
		query: null,
		filter: true,
		emptytext: $.LL("No content"),
        onLoad: function () { }
    };

    if (arguments.length) {
        $opts = $.extend($opts, arguments[0]);
        $opts.onLoad = arguments[0].onLoad;
    }
	
	$opts.container.addClass("genericquery");
	
	if (!$opts.query) {
		$opts.container
			.html($opts.emptytext)
			.css({
				"font-size": "1.3em",
				"margin": "0.65em 0.5em"
			});
			
		return;
	}
	
	if (!$opts.type) {
		$otps.type = "full";
	}
	
	var script = "GWE_ServiceHandlers.";
	
	if ($opts.type.match(/structure/i)) {
		script += "GetGenericQueryStructure";
	}
	else if ($opts.type.match(/data/i)) {
		script += "GetGenericQueryData";
	}
	else {
		script += "GetGenericQueryFull";
	}

	$.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		url: initQisCsharpMethod.call($obj, script),
		data: window.JSON.stringify({
			objectAttributes: [
				{Name:"GenericQuery", Format:"PlainText", Value: $opts.query.split("/")[0]},
				{Name:"RefObj", Format:"PlainText", Value: $obj.Data.ObjId},
				{Name:"Filter", Format:"PlainText", Value: $opts.filter}
			]
		})
	})
	.done(function (data) {
		if (!isValidJsonData.call(null, data, script)) {
			data = "";
		}
		if(!data || data.length <= 0) {
			$opts.container
				.html($opts.emptytext)
				.css({
					"font-size": "1.3em",
					"margin": "0.65em 0.5em"
				});
			return;
		}
		
		var $data = data.length ? $.parseJSON(data) : {};
		$opts.onLoad($data);
	})
	.fail(function(){
		$.browser.error("Failed getting generic query")
		$opts.container
			.html($opts.emptytext)
			.css({
				"font-size": "1.3em",
				"margin": "0.65em 0.5em"
			});
	});
}

RepositoryObject.prototype.ObjectDialog = function () {
	var $obj = this;
	var args = arguments;
	
	if (!$obj.TemplateDefinition) {
		return $obj.TemplateDefinitions(function(){
			$obj.ObjectDialog.apply($obj, args);
		})
	}

    var custom = args[0] || null;
    var tmplt = args[1] || null;
    var attrib = args[2] || null;
    var width = args[3] || ($obj.TemplateDefinition.Width || $QepSettings.Definitions.FancyBox.Width);
    var height = args[4] || ($obj.TemplateDefinition.Height || $QepSettings.Definitions.FancyBox.Height);
    var objid = args[5] || $obj.Data.ObjId;
    var repid = args[6] || $QisSession.RepId;
    var confid = args[7] || $QisSession.ConfId;
    var callb = args[8] || $.noop;
    var gweAttr = args[9] || null;

    var href = $QisSession.HTMLBase + "/ObjectDialog.aspx?JS=True";
    
    if (!tmplt) {
        href += "&Id=qlm://" + objid + "/?repository=" + repid;
    }
    else {
        href += "&NewObjectTemplate=" + tmplt;
		if (attrib && attrib.substring(0, 1) == "&") {
			href += "&ObjId=" + objid + attrib;
		} 
		else {
			if (attrib) {
				href += "&AssociatedWith=qlm://" + objid;
				href += "&Attribute=" + attrib;
			}
			else
				href += "&ObjId=" + objid + "/?repository=" + repid;
		}
	}
	
    if (custom) {
        href += "&Customization=" + custom;
    }
	
    if (gweAttr) {
        href += "&gweAttr=" + gweAttr;
    }
	
    href += "&RepId=" + repid;
    href += "&ConfId=" + confid;
   
    var $ifrm = $("iframe.objectdialog");

    if ($ifrm.length) {
        $ifrm.remove();
    }
	
    $ifrm = $("<iframe />")
        .attr("id", "objectdialog")
        .attr("class", "objectdialog iframeview")
        .attr("name", "iframeview fancybox")
        .attr("width", "100%")
        .attr("height", "100%")
        .attr("seamlss", "seamless")
        .load(function () {
            if ($(this).data("loaded")){
                return;
            }
			
			$ifrm = $(this);
			
            if (!$ifrm.contents().text()) {
                return;
            }
						
			$qdlg = $ifrm.data("loaded", true)
				.contents().find("body");
			
			if ($qdlg.find(".unhandledException").text()) {
				$(".fancybox-outer").append($("<button/>")
					.html("&nbsp;" + $.LL("Close") + "&nbsp;")
					.button()
					.on($.browser.click, function(){
						$ifrm.attr("src", "about:blank"); 
						$.async(function(){$.fancybox.close();}, 100);
					})
					.css({
						"line-height": "2em",
						"font-size": "0.9em",
						"position": "absolute",
						"bottom": "15px",
						"right": "20px",
						"width": "80px"
					}));
			}
			
			// Repaint all Telerik controls to update their sizes.
			// Some controls may not be properly sized, because DOM is loaded before it starts rendering (e.g RadGrid).
			// The later is caused by the way Object Dialog is loaded here.
			if($ifrm.get(0) && $ifrm.get(0).contentWindow)
			{
				repaintRadControls($ifrm.get(0).contentWindow);
			}
			
			width = $ifrm.contents().width() || $qdlg.data("width") + 5;
			height = $ifrm.contents().height() || $qdlg.data("height") + 110;
			$odlg.width(width).height(height);
			$.fancybox.update();
			$.async(function(){
				$.overlay(false);
			}, 100);

		});

	var $odlg = $("<div />")
		.addClass("webform")
		.width(600)
		.height(400)
		.css("overflow", "hidden")
		.append($ifrm);

	$.async(function(){
		$ifrm.attr("src", href);
		$.fancybox($odlg, {
			'modal' : true, 
			'scrolling' : 'auto',
			afterLoad: function(){
				$("body").addClass("modal-window");
				$.overlay({
					loader: true,
					text: $.LL("Loading form data") + "...",
					transparent: true
				});
				
			},
			afterClose: function(){
				$("body").removeClass("modal-window");
				callb.call();
			}
		});
	});
}

RepositoryObject.prototype.DiagramEditor = function() {
	var $obj = this;
	
	if ($(".governanceactions").length) {
		$.async(function(){
			$(".governanceactions").empty()
				.ajaxLoaderText(true, $.LL("Loading diagram editor") + "...");
		}, 100);
	}
	
	if (!$obj.Type.match(/diagram/i)) {
		return $.async(function(){
			$obj.GovernanceActions();
		}, 1500)
	}
	else {
		initDiagramEditor($obj);
	}
}

RepositoryObject.prototype.sortObjectList = function () {
    var $list = arguments[0];

    if (!$list.length) {
        return $list;
    }

    var $opts = {
		sortAttribute: "Name",
        sortOrder: "Ascending"
    }

    if (arguments[1]) {
        $opts = $.extend($opts, arguments[1]);
    }
	
	if (!$opts.sortAttribute || !$opts.sortOrder) {
		return $list;
	}

    $list.sort(function ($obj1, $obj2) { return ($obj1.Data[$opts.sortAttribute].toLowerCase() < $obj2.Data[$opts.sortAttribute].toLowerCase()) ? -1 : 1; });

    if ($opts.sortOrder.toLowerCase().indexOf("desc")>-1) {
        $list.reverse();
    }
    
    return $list;
}

RepositoryObject.prototype.TemplateDefinitions = function() {
	if (!this.Type) {
		$.extend(true, this, $TemplateSettings[this.Data.Template]);
	}
	
	var $obj = this;
	var callback = arguments[0] ? arguments[0] : $.noop;

	if ($QisSession.Init[$obj.Data.ObjId + "-defs"]) {
		$.async(function(){
			if ($QisSession.Init[$obj.Data.ObjId + "-defs"]) {
				return;
			}
			$.async(false, this);
			$obj.TemplateDefinition = $.pageCache("get", "def" + $obj.Data.ObjId);
			callback.call($obj, $obj.TemplateDefinition);
		}, 50, true);
		return;
	}
	
	if ($.pageCache("get", "def" + $obj.Data.ObjId, true)) {
		$obj.TemplateDefinition = $.pageCache("get", "def" + $obj.Data.ObjId);
		callback.call($obj, $obj.TemplateDefinition);
		return;
	}
	
	if ($.browserCache("get", "def" + $obj.Data.ObjId)) {
		$obj.TemplateDefinition = $.browserCache("get", "def" + $obj.Data.ObjId);
		$.pageCache("set", "def" + $obj.Data.ObjId, $obj.TemplateDefinition, false);
		callback.call($obj, $obj.TemplateDefinition);
		return;
	}
			
	if (!$obj.Data.Template) {
		return $obj.audit(function(valid){
			if (valid) {
				$obj.TemplateDefinitions.call($obj, callback);
			}
		}, false, true);
	}
	
	$QisSession.Init[$obj.Data.ObjId + "-defs"] = 1;
	
	if (!$obj.TemplateSettings.length) {
		$obj.TemplateDefinition = { Target: "Excluded" };
		$.pageCache("set", "def" + $obj.Data.ObjId, $obj.TemplateDefinition, false);
		delete $QisSession.Init[$obj.Data.ObjId + "-defs"];
		callback.call($obj, null);
		return;
	}
		
	var processTemplateSettings = null;

	(processTemplateSettings = function(){
		if ($.keyCount($obj.TemplateDefinition)) {
			delete $QisSession.Init[$obj.Data.ObjId + "-defs"];
			callback.call($obj, $obj.TemplateDefinition);
			return;
		}
		
		var index = arguments[0] || 0;
		if (index >= $obj.TemplateSettings.length) {
			$obj.TemplateDefinition = { Target: "Excluded" };
			$.pageCache("set", "def" + $obj.Data.ObjId, $obj.TemplateDefinition, false);
			delete $QisSession.Init[$obj.Data.ObjId + "-defs"];
			callback.call($obj, $tdef);
		}
		
		var sfile = $obj.TemplateSettings[index];
		var $tdef = $TemplateDefinitions[sfile];
		
		if (!$tdef.RelevantFor.length && !$.keyCount($tdef.ApplyConditions)) {
			$obj.TemplateDefinition = $tdef;
			$.pageCache("set", "def" + $obj.Data.ObjId, $tdef, false);
			delete $QisSession.Init[$obj.Data.ObjId + "-defs"];
			callback.call($obj, $tdef);
			return;
		}
		
		if ($tdef.RelevantFor.indexOf($obj.Data.ObjId) > -1) {
			$obj.TemplateDefinition = $tdef;
			$.pageCache("set", "def" + $obj.Data.ObjId, $tdef, false);
			delete $QisSession.Init[$obj.Data.ObjId + "-defs"];
			callback.call($obj, $tdef);
			return false;
		}
		
		if (!$.keyCount($tdef.ApplyConditions)) {
			return processTemplateSettings(++index);
		}
		
		var isValid = true;
		var attrList = [];
		
		$.each($tdef.ApplyConditions, function(idx, condition){
			if (condition.Attribute) {
				attrList.push(condition.Attribute.match(/objid/i) ? 
					"ObjId" : 
					condition.Attribute);
			}
		});
		
		$obj.info(attrList.unique(), function() {
			$.each($tdef.ApplyConditions, function(idx, condition){
				if (!$obj.ValidCondition(condition)) {
					isValid = false;
					return false;
				}
			});
			
			if (isValid) {
				$obj.TemplateDefinition = $tdef;
				$.pageCache("set", "def" + $obj.Data.ObjId, $tdef, false);
				delete $QisSession.Init[$obj.Data.ObjId + "-defs"];
				callback.call($obj, $tdef);
				return;
			}
						
			return processTemplateSettings(++index);
		});
	}).call(null, 0);
}

RepositoryObject.prototype.ValidCondition = function() {
	var condition = arguments[0] || false;
	if (!condition) {
		return true;
	}

	var isValid = false;
	
	if (typeof condition == "string") {
		var $c = {
			Attribute: null,
			Operation: null,
			Value: null
		};
		
		var temp = condition.split(/==|:=|!=|>|</);
		
		if (temp.length < 2) {
			temp = condition.split("=");
		}
		
		$c.Attribute = temp[0];
		$c.Value = temp[1] || "";
		
		if (condition.match("==")) {
			if (temp[1]) {
				$c.Operation = "Equal";
			}
			else {
				$c.Operation = "IsEmpty";
			}
		}
		else if (condition.match("!=")) {
			if (temp[1]) {
				$c.Operation = "NotEqual";
			}
			else {
				$c.Operation = "NotEmpty";
			}
		}
		else if (condition.match(":=")) {
			$c.Operation = "Contains";
		}
		else if (condition.match(">")) {
			$c.Operation = "GreaterThan";
		}
		else if (condition.match(">")) {
			$c.Operation = "LessThan";
		}
		else if (condition.match("=")) {
			if (temp[1]) {
				$c.Operation = "Equal";
			}
			else {
				$c.Operation = "IsEmpty";
			}
		}
		
		condition = $c;
	}

	var $obj = this;
	
	var $opts = {
		Attribute: null, 
		Operation: null,
		Value: null
	};
	
	$.extend(true, $opts, condition);
	$opts.Attribute = $opts.Attribute.replace(/CurrInst->/i, "");

	if ($opts.Attribute.match(/objid/i)) {
		$opts.Attribute = "ObjId";
	}
		
	if (!($opts.Attribute in $obj.Data)) {
		return isValid;
	}
	
	var reqval = $.trim($opts.Value);
	var attval = $obj.attr($opts.Attribute).text();
	
	if (!attval) {
		return !reqval;
	}
	
	if ($opts.Attribute.match(/AuditMD|AuditCD/)) {
		attval = $obj.Data[$opts.Attribute];
	}
	else if ($obj.Data[$opts.Attribute].Format === "Date"){
		attval = $obj.Data[$opts.Attribute].Value;
	}
	
	switch ($opts.Operation) {
		case "Equal": 
			if (attval == reqval) {
				isValid = true;
			}
			break;
			
		case "BeginsWith":
			if (attval.indexOf(reqval)==0) {
				isValid = true;
			}
			break;
			
		case "GreaterThan":
			if (attval > reqval) {
				isValid = true;
			}
			break;
			
		case "LessThan":
			if (attval < reqval) {
				isValid = true;
			}
			break;
			
		case "Contains":
			if (attval.indexOf(reqval)>-1) {
				isValid = true;
			}
			break;
			
		case "NotEqual":
			if (attval != reqval) {
				isValid = true;
			}
			break;
			
		case "IsEmpty":
			if (!attval) {
				isValid = true;
			}
			break;
			
		case "IsNotEmpty":
			if (attval) {
				isValid = true;
			}
			break;
			
		default: break;
	}
	
	return isValid;
}

RepositoryObject.prototype.GrabLock = function(){
	var calb = arguments[0] || $.noop;
	var $obj = arguments[1] || $$;
	
	if ($obj.Data.RevisionId) {
		return $obj.audit(function(valid){
			if (valid) {
				$obj.GrabLock(calb, $obj);
			}
		});
	}
	
	var oid = $obj.Data.ObjId + "/" + $obj.Data.RevisionId;
	
	var wsurl = initQisCsharpMethod.call($obj, "ServiceHandlers.GrabLock"); 
	var parms = window.JSON.stringify({ 
		objectAttributes:[	
			{Name:"sys_ObjectId", Format:"PlainText", Value: oid}
		]
	});
	
	$.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		url: wsurl,
		data: parms
	})
	.done(function (data) {
		if (!isValidJsonData.call(null, data, "GrabLock")) {
			return;
		}
	});
}

RepositoryObject.prototype.SetLock = function(){
	var calb = arguments[0] || $.noop;
	var $obj = arguments[1] || $$;
	
	if ($obj.Data.RevisionId) {
		return $obj.audit(function(valid){
			if (valid) {
				$obj.SetLock(calb, $obj);
			}
		});
	}
	
	var oid = $obj.Data.ObjId + "/" + $obj.Data.RevisionId;
	
	var wsurl = initQisCsharpMethod.call($obj, "ServiceHandlers.SetLock"); 
	var parms = window.JSON.stringify({ 
		objectAttributes:[	
			{Name:"sys_ObjectId", Format:"PlainText", Value: oid}
		]
	});
	
	$.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		url: wsurl,
		data: parms
	})
	.done(function (data) {
		if (!isValidJsonData.call(null, data, "SetLock")) {
			return;
		}
	});
}

RepositoryObject.prototype.ReleaseLock = function(){
	var calb = arguments[0] || $.noop;
	var $obj = arguments[1] || $$;
	
	if ($obj.Data.RevisionId) {
		return $obj.audit(function(valid){
			if (valid) {
				$obj.ReleaseLock(calb, $obj);
			}
		});
	}
	
	var oid = $obj.Data.ObjId + "/" + $obj.Data.RevisionId;
	
	var wsurl = initQisCsharpMethod.call($obj, "ServiceHandlers.ReleaseLock"); 
	var parms = window.JSON.stringify({ 
		objectAttributes:[	
			{Name:"sys_ObjectId", Format:"PlainText", Value: oid}
		]
	});
	
	$.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		url: wsurl,
		data: parms
	})
	.done(function (data) {
		if (!isValidJsonData.call(null, data, "ReleaseLock")) {
			return;
		}
	});
}

_$$_ = function () {
	if (History.enabled) {
		NProgress.configure({
			minimum: 0.15,
			speed: 750,
			trickle: true
		}).start();
	}
	
	$$ = new RepositoryObject(arguments[0]);
	$$.cache("get");
	$$.init(function(){
		$.async(function(){
			if (History.enabled) {
				NProgress.set(0.45);
			}
		}, 100);
		$$.TemplateDefinitions(function() {
			if (History.enabled) {
				if (!$.keyCount(History.getState().data)) {
					History.replaceState($$, $$.Data.Name, window.location.search);
				}
			}
			var settings = $$.TemplateDefinition["SettingsFile"];
			if  ($QepSettings.SettingsFile === settings) {
				$.initTemplateSettings($$, function() {
					$.initPageContent(null, true, false);
				});
			}
			else {
				$.initTemplateFields();
			}
		});
	}, true);
}

initQisObject = function () {
    var oid = getUrlParams("oid").split("#")[0];
    var rid = getUrlParams("rid").split("#")[0];
	var pid = getUrlParams("pid") || "";
	
	if (oid) {
		return _$$_({ObjId: oid, RepId: rid||$QisSession.RepId});
	}
	else {
		if (($QepSettings.PersonalPage.Enable
			&& (!$QepSettings.StartPage 
			|| !$QepSettings.PersonalPage.Content))
		|| pid.match(/personal/)) {
			loadPersonalPage();
		}
		else {
			_$$_($QepSettings.StartPage);
		}
	}
}

isObjectExcluded = function() {
	var $obj = arguments[0];
	var filter = arguments[1];
	
	if (!$obj) {
		return true;
	}
	
	if (!filter) {
		return false;
	}
	
	if (typeof filter == "string") {
		filter = filter.split(/,|;/g);
	}
	
	var checkTemplate = function($o, t){
		var template = $o.Data.Template.toLowerCase();
		if ( t.indexOf("*") > -1) {
			var templ = t.split("*")[0];
			if (template.indexOf(templ.toLowerCase()) > -1) {
				return true;
			}
		}
		else {
			if (template == t.toLowerCase()) {
				return true;
			}
		}
		return false;
	}
	
	var excluded = false;
	
	$.each(filter, function() {
		if (this.substring(0,1) == "-") {
			if (checkTemplate($obj, this.substring(1))) {
				excluded = true;
				return false;
			}
		}
		else {
			if (checkTemplate($obj, this)) {
				excluded = false;
				return false;
			}
			else {
				excluded = true;
			}
		}
	});
	
	return excluded;
}

setSessionVariables = function () {
    $.cookie("UserFullName", $QisSession.UserInfo.FullName);
    $.cookie("UserLogin", $QisSession.UserInfo.Login);
    $.cookie("UserObjId", $QisSession.UserInfo.ObjId);
    $.cookie("UserRole", $QisSession.UserInfo.Role);
    $.cookie("UserAltRoles", $QisSession.UserInfo.AltRoles);
    $.cookie("LangCode", $QisSession.LangCode);
}

clearSessionVariables = function () {
    $.removeCookie("UserSid", { path:'/' });
    $.removeCookie("UserFullName");
    $.removeCookie("UserLogin");
    $.removeCookie("UserObjId");
    $.removeCookie("UserGroup");
    $.removeCookie("UserRole");
    $.removeCookie("UserAltRoles");
    $.removeCookie("MangoChatSessionCookie", {path: "/"});
    $.removeCookie("DialogStates", { path: "/" });
    $.removeCookie("_ResponsiveTop_");
    $.removeCookie("_ResponsiveLeft_");
	
    $.browserCache("clear", true);
	
    $QisSession.UserInfo = {};
}

initCommandDialog = function(){
	var $cnt = $("<div/>").width(500)
		.append("<h3>" + $.LL("QualiWare Command Language") + "</h3>")
		.append("<div><textarea spellcheck=\"false\">" + $.LL("your qcl code") + "..." + "</textarea></div>");
		
	$btn = $("<div/>")
		.append("<button class=\"btnRun\">&nbsp;" + $.LL("Run") + "&nbsp;</button>")
		.append("<button class=\"btnCancel\">" + $.LL("Cancel") + "</button>")
		.css({
			"text-align": "right",
			"margin-top": "1em"
		})
		.appendTo($cnt);
		
	$cnt
		.find("h3").css({
			"margin-bottom": "0.8em", 
			"font-size": "1.3em",
			"padding-bottom": "0.4em",
			"border-bottom": "solid 1px #000"
		}).end()
		.find("textarea").each(function() {
			$(this)
				.data("default", this.value)
				.css({
					"font-family": "consolas, verdana",
					"width": "485px",
					"height": "150px"
				});
		})
		.focus(function() {
			if (!$(this).data("edited")) {
				this.value = "";
			}
		})
		.blur(function() {
			if (!$(this).data("edited")) {
				this.value = $(this).data("default");
			}
		})
		.bind("input propertychange", function() {
			$(this).data("edited", this.value != "")
		});
		
	$btn.children(".btnRun").each(function(){
		$(this).button().on($.browser.click, function(){
			var $txt = $cnt.find("textarea").attr("disabled", true);
			var val = $txt.val();
			
			if (val){
				$cnt.find("h1").html($.LL("Executing") + "...");
				initQclEngine(val);				
			}
			else {
				$.fancybox.close();
			}
		})
		.css({
			"line-height": "2em",
			"font-size": "0.9em",
			"margin-left": "4px"
		});
	})

	$btn.children(".btnCancel").each(function(){
		$(this).button().on($.browser.click, function(){
			$.fancybox.close();
		})
		.css({
			"line-height": "2em",
			"font-size": "0.9em",
			"margin-left": "4px"
		});
	});
		
	$.fancybox($cnt, {modal: true});
}

initQclEngine = function() {
	if (!arguments.length) {
		return;
	}

	var command = arguments[0];
	var callback = arguments[1] || null;
	var type = arguments[2] || "QCLServerEngine";
	var $obj = $$;
		
	if (this instanceof RepositoryObject) {
		$obj = this;
	}
	
	var $engine = $QclEngine.Data ? $QclEngine.Data[type] : {};
	
	if (!$engine.Id) {
		$.overlay(false);
		$.fancybox($("<div/>").html("<h1>" + $.LL("Warning") + "</h1><div>" + $.LL("QCL Engine is not available") + "</div>")
			.width(350).height(80));
		return;
	}

	if (!command){
		$.fancybox($("<div/>").html("<h1>" + $.LL("Warning") + "</h1><div>" + $.LL("Empty QCL command can not be used for QCL Engine") + "</div>")
			.width(350).height(80));
		return;
	}
	
	command += command.substring(command.length - 1) === ";" ? "" : ";";
	
	var cmd_pre = "SetWorkingLanguageCode(\"" + $QisSession.LangCode + "\"); QepSwitchConfiguration(\"" + $QisSession.ConfId + "\");\n"
		+ "CurrInst = ObjID2Entity(\"" + $obj.Data.ObjId+ "\", \"" + $obj.Data.RepId+ "\");\n";
	
	if ($obj.Type === "Diagram") {
		cmd_pre += "CurrGraph = LastActiveGraph = ObjID2Entity(\"" + $obj.Data.ObjId+ "\", \"" + $obj.Data.RepId+ "\");\n";
	}
	else if ($$.Type === "Diagram") {
		cmd_pre += "\nCurrGraph = LastActiveGraph = ObjID2Entity(\"" + $$.Data.ObjId+ "\", \"" + $$.Data.RepId+ "\");\n";
	}
	
	var cmd_post = "\nDiscardEntities();";
	
	var wsurl = initQisCsharpMethod.call($obj, "ServiceHandlers.ExecuteQcl");
	var parms = window.JSON.stringify({ 
		objectAttributes:[
			{Name:"InstanceId", Format:"PlainText", Value: $engine.Id},
			{Name:"Command", Format:"PlainText", Value: cmd_pre + command + cmd_post}
		] 
	});
	
	$.ajax({
		type: 'POST',
		dataType: 'JSON',
		contentType : 'application/json',
		cache: false,
		url: wsurl,
		data: parms
	})
	.done(function (data) {
		if (!isValidJsonData.call(null, data, "ExecuteQcl")) {
			data = "";
		}
		
		var $data = $.parseJSON(data);
		var $cnt = $("<div />").width(600);
		
		if ($data.Messages) {
			$data.Messages = $data.Messages.replace(/Warning: Suspicious assignment to CurrGraph/gi, "");
			$data.Messages = $data.Messages.replace(/Warning: Suspicious assignment to CurrInst/gi, "");
			$data.Messages = $data.Messages.replace(cmd_pre, "");
			$data.Messages = $data.Messages.replace(cmd_post, "");
			if ($data.Messages == "\n") {
				$data.Messages = null;
			}
		}
		
		if (typeof callback === "function") {
			return callback.call($data);
		}
		
		$cnt.append("<h1>" + $.LL("Result:") + "</h1>");
	
		if ($data.Duration) {
			$cnt.append("<div style=\"margin-top:-5px;\"><b style=\"color:green;\">" + $.LL("Duration:") + "</b> " + $data.Duration/1000 + "sec.</div>");
		}
		
		if (!$data.Messages && $data.Result) {
			$cnt.append("<br/><div><b>" + $.LL("Result:")+ "</b><br/>" + $data.Result + "</div>");
		}
		else {
			$cnt.append("<hr/>" + $data.Messages);
		}
		
		if ($cnt.find("table").length) {
			$cnt.width(Math.min($(window).width()*0.75, 1280));
		}
					
		$.fancybox($cnt);
	})
	.fail(function (jqxhr, textStatus, error) {
		handleRequestError(jqxhr, textStatus, error, wsurl, parms);
	})
}

initQisWebControls = function() {
	$.browser.info("Initializing Telerik components...");
	var callback = arguments[0] || $.noop;	
	var url = $QisSession.HTMLBase + "/QueryResultView.aspx"
			+ "?Templates=Icon"
			+ "&ConfId=" + $QisSession.ConfId
			+ "&RepId=" + $QisSession.RepId;
				
	var $cnt = $("<div/>").addClass("init-controls").css({
		"position": "absolute",
		"left": "-9999px",
		"width": "600px",
		"height": "800px"
	})
	.appendTo("body");
	
	var $ifrm  = $("<iframe width=\"100%\" height=\"100%\" src=\"" + url + "\"/>")
		.appendTo($cnt);
	
	callback.call();

	$ifrm.load(function(){
		try {
			if (!($ifrm.contents()) && !$ifrm.contents().text()) {
				return;
			}
		}
		catch(e) { 
			return; 
		}
		if ($ifrm.attr("src").match(/about:blank/i)) {
			return;
		}
		$.browser.info("Telerik components initialized.");
		$.async(function(){
			$ifrm.attr("src", "about:blank");
			$.async(function(){
				if ($QisSession.Addons.SBW) {
					initQisChartControls();
				}
				else {
					$cnt.remove();
				}
			}, 250);
		}, 2000);
	});
}

initQisChartControls = function() {
	$.browser.info("Initializing DevXpress components...");

	var url = $QisSession.HTMLBase + "/BusinessChartOnObject.aspx"
			+ "?Id=qlm://90d9b9ef-a189-4743-b2dc-2d6b10f09f23/?repository=" + $QisSession.RepId
			+ "&RefObj=qlm://" + $QisSession.UserInfo.ObjId+ "/?" + $QisSession.RepId
			+ "&ConfId=" + $QisSession.ConfId
			+ "&RepId=" + $QisSession.RepId;
	
	var $cnt = $("div.init-controls");
	
	if ($cnt.length) {
		$cnt.empty();
	}
	else {
		$cnt = $("<div/>").addClass("init-controls").css({
			"position": "absolute",
			"left": "-9999px",
			"width": "600px",
			"height": "800px"
		})
		.appendTo("body");
	}
	
	var $ifrm  = $("<iframe width=\"100%\" height=\"100%\" src=\"" + url + "\"/>")
		.appendTo($cnt);
	
	
	$ifrm.load(function(){
		try {
			if (!($ifrm.contents()) && !$ifrm.contents().text()) {
				return;
			}
		}
		catch(e) { 
			return; 
		}
		if ($ifrm.attr("src").match(/about:blank/i)) {
			return;
		}
		$.browser.info("DevXpress components initialized.");
		$.async(function(){
			$ifrm.attr("src", "about:blank");
			$.async(function(){
				$cnt.remove();
			}, 250);
		}, 1000);
	});
}

initQisSession = function () {
    $QisSession.UserInfo.Sid = $.cookie("UserSid");
	
	if (!$QisSession.UserInfo.Sid) {
		return endQisSession(location.href);
	}
		
    var startQisSession = function() {
		var stopExecution = false;
		
		if (!$.cookie("UserFullName") || !$.cookie("UserRole") 
		|| ($QisSession.ConfId != $.browserCache("get", "persistent-Configuration"))) {
			$.browserCache("clear");
			$.fancybox.close();
			$("body").addClass("fancy-lock");
			
			var title = "<span>" + $.LL("Connecting") + "</span>";
			var caption = $.LL("Repository:") + " " + $QisSession.RepName;
			
			if ($QisSession.ConfId != $QisSession.BaseConfId ) {
				caption += " [" + $QisSession.ConfName + "]";
			}
			
			var wsurl = initQisCsharpMethod("ServiceHandlers.GetUserInfo");
			var parms = window.JSON.stringify({objectAttributes:[]});
			
			var $jbar = $.jbar({
				bgcolor: "#fcfcfc",
				opacity: 0.7,
				title: title,
				caption: caption,
				time: -1
			});
			
			var $cnt = $jbar.find(".jbar-content").css("margin-top", "10px");
			var left = $cnt.find("span").position().left;
			
			$cnt.empty()
				.css("text-align", "left")
				.loadingText($.LL("Connecting"))
					.find(".loadingtext").css({
						"position": "relative",
						"left": left
					});

			$.ajax({
				type: 'POST',
				dataType: 'JSON',
				contentType : 'application/json',
				url: wsurl,
				data: parms
			})
			.done(function (data) {				
				if (!isValidJsonData.call(null, data, "GetUserInfo")) {
					data = "";
				}
				
				var $info = data.length ? $.parseJSON(data) : null;
				
				if (!data) {
					return;
				}
				
				$.browserCache("set", "persistent-Configuration", $QisSession.ConfId);
				
				$cnt.css("text-align", "center").html($.LL("Connected"));
				
				$QisSession.UserInfo.FullName = $info.Name;
				$QisSession.UserInfo.Login = $info.Login;
				$QisSession.UserInfo.ObjId = $info.ObjId;
				$QisSession.UserInfo.Role = $info.DefaultRole;
				$QisSession.UserInfo.AltRoles = $info.AltRoles;
				$QisSession.LangCode = $info.LangCode;
				
				setSessionVariables();
				
				$.each($QisSession.FncQueue, function(idx, fnc) {
					if (typeof fnc === "function") {
						stopExecution = stopExecution || fnc.call();
						$QisSession.FncQueue.splice(idx, 1);
					}
				});

				$QisSession.FncQueue = [];
				$QisSession.Connected = false;
				
				if (!stopExecution) {
					initQisObject();
				}
				
				var pageInitialized = function() {
					$.async(function(){
						$jbar.animate({"margin-top":"-110px"}, 750, function(){
							$(".jbar").remove();
							$("body").removeClass("fancy-lock");
						});
					}, 2000, false, true);
				};
				
				if (!$.cookie("WebControlsInit")) {
					if (!History.enabled) {
						$cnt.empty().css("text-align", "left")
							.loadingText($.LL("Initializing Web Controls"),"", 500, 5)
								.find(".loadingtext").css({
									"position": "relative",
									"left": parseInt(left) - 80 + "px"
								});
								
						initQisWebControls( function(){
							$.cookie("WebControlsInit", 1);
							if (!History.enabled) {
								$cnt.css("text-align", "center").html($.LL("Done"));
								pageInitialized();
							}
						});
					}
					else {
						$cnt.css("text-align", "center").html($.LL("Done"));
						$.cookie("WebControlsInit", 1);
						initQisWebControls();
						pageInitialized();
					}
					
				}
				else {					
					pageInitialized();
				}
			})
			.fail(function (jqxhr, textStatus, error) {
				handleRequestError(jqxhr, textStatus, error, wsurl, parms);
			});
		}
		else {		
			$QisSession.UserInfo.FullName = $.cookie("UserFullName");
			$QisSession.UserInfo.Login = $.cookie("UserLogin");
			$QisSession.UserInfo.ObjId = $.cookie("UserObjId");
			$QisSession.UserInfo.Role = $.cookie("UserRole");
			$QisSession.UserInfo.AltRoles = $.cookie("UserAltRoles");

			$.each($QisSession.FncQueue, function(idx, fnc){
				if (typeof fnc === "function") {
					stopExecution = stopExecution || fnc.call();
					$QisSession.FncQueue.splice(idx, 1);
				}
			})
			
			if (!stopExecution) {
				initQisObject();
			}
		}
	}
	
	var isLoggedIn = function (poll, interval) {
		var wsurl = initQisWebService({
			type: "isloggedin",
			parameters: $QisSession.UserInfo.Sid
		});
		
		$.getJSON(wsurl)
			.done( function(data) {
				if (!isValidJsonData.call(null, data, "isloggedin")) {
					data = "";
				}
				
				var result = data.length ? $.parseJSON(data) : null;
				if (result) {
					if (poll && !$QisSession.Offline) {
						$.browser.info("Session life time extended...");
					}
					else {
						if ($QisSession.Offline) {
							$QisSession.Offline = false;
						}
						startQisSession();
					}
				}
				else {
					endQisSession(location.href);
				}
			})
			.fail(function (jqxhr, textStatus, error) {
				$.async(false, interval);
				$.async(false);
				handleRequestError(jqxhr, textStatus, error, wsurl);
				$.async(function(){
					var interval = this;
					$.ajax({
						type : "GET",
						url : wsurl,
						timeout: 3000,        
						dataType: "json",
						success : function (response, textStatus, xhr) {
							$.async(false, interval);
							initQisSession();
						},
						error: function (jqxhr, textStatus, error) {           
							if(textStatus.match(/timeout/i)) {
								$.browser.error("Network connection lost... - trying to reconnect in 5 seconds.");
							}
						}
					});
				}, 5000, true);
			});
	}
	
	if ($.cookie($QisSession.RepId + "-Active")) {
		startQisSession();
	}
	else {
		$.cookie($QisSession.RepId + "-Active", 1);
		isLoggedIn();
	}
	
	$.async(function(){
		isLoggedIn(true, this);
	}, 60000, true, true);
	
}

endQisSession = function () {
    var returnUrl = arguments[0] || $QisSession.HTMLLocation;
    
    var singleSignOn = true;
	
	if (typeof arguments[1] !== "undefined")  {
		singleSignOn = arguments[1];
	}
	
    var baseUrl = $QisSession.HTMLBase 
				+ "/Login.aspx?EnableSingleSignOn=" + singleSignOn + "&ReturnUrl=" + escape(returnUrl);

    if (!$.cookie("UserSid")) {
        return location.replace(baseUrl);
    }

	clearSessionVariables();

    var wsurl = initQisWebService({
		type: "logout",
		parameters: $QisSession.UserInfo.Sid
	});
		
    $.getJSON(wsurl).always(function () {
		location.replace(baseUrl);
	})
}

initQisAuthentication = function() {
	var $opts = this;
	var callbackOnSuccess = arguments[0] || $.noop;	
	var callbackOnFail = arguments[1] || $.noop;
	var invalidLoginText = arguments[2] || null;
	
	$.LoginForm(function(){
		if ($opts.RequiresFullAuthentication 
		&& !$opts.AllowChangeUser) {
			var regex = new RegExp(this.username, "i");
			if (!$QisSession.UserInfo.Login.match(regex)) {
				invalidLoginText = $.LL("Invalid username or password");
				initQisAuthentication.call($opts, callbackOnSuccess, callbackOnFail, invalidLoginText);
			}
		}
		
		var wsurl = $QisSession.HTMLBase + "/QisWebService.svc/json/login";
		$.browser.log(wsurl);
		$.ajax({
			type: "POST",
			url: wsurl,
			data: JSON.stringify({
				authority: this.domain, 
				authorityKind: this.authority ? "Domain" : "Local", 
				login: this.username, 
				password: this.password
			}),
			contentType: "application/json; charset=utf-8",
			dataType: "json",
			async: false
		})
		.done(function(data){
			$.fancybox.close();
			if (!isValidJsonData.call(null, data, "login")) {
				return;
			}
			
			var sid = $.parseJSON(data);
			callbackOnSuccess.call(sid);
			
			$.async(function(){
				if (!$opts.AllowChangeUser) {
					$.getJSON(initQisWebService({
						type: "logout",
						parameters: sid
					}));
				}
			}, 300000);
		})
		.fail(function(jqxhr, textStatus, error){
			var text = String(jqxhr.responseText);
			if (text.indexOf("Invalid user")) {
				invalidLoginText = $.LL("Invalid username or password");
				initQisAuthentication.call($opts, callbackOnSuccess, callbackOnFail, invalidLoginText);
			}
			else {
				handleRequestError(jqxhr, textStatus, error);
				callbackOnFail.call();
			}
		});
	}, callbackOnFail, invalidLoginText, $opts);
}

initQisWebService = function () {
    if (!arguments.length) {
        return null;
    }

    if (!$QisSession.UserInfo.Sid) {
        $.removeCookie("UserSid", { path:'/' });
        return null;
    }
	
	var $opts = {
		objid: null,
		revid: null,
		confid: $QisSession.ConfId,
		repid: $QisSession.RepId,
		method: null,
		relation: null,
		field: null,
		type: "qis",
		format: "json",
		wsfunction: "",
		parameters: ""
	};
	
	if (this instanceof RepositoryObject) {
		if (this.Data.RepId) {
			$opts.repid = this.Data.RepId;
		}
		if (this.Data.ConfId) {
			$opts.confid = this.Data.ConfId;
		}
	}
	
	if (typeof arguments[0] === "string") {
		$opts.objid = arguments[0];
	}
	else if (arguments[0]) {
		$.extend(true, $opts, arguments[0]);
	}
	
	if (!$opts.repid) {
		$opts.repid = $QisSession.RepId;
		$opts.confid = $QisSession.ConfId;
	}
	
	if ($opts.repid != $QisSession.RepId) {
		var $data = getRepositoryInfo($opts.repid);
		if ($opts.repid != $data.RepId) {
			return null;
		}
		$opts.confid = $data.ConfId;
	}
	else if ($opts.confid != $QisSession.ConfId) {
		var $data = getConfigurationInfo($opts.confid);
		if ($opts.confid != $data.Id) {
			return null;
		}
	}

    if (!$opts.parameters) {
        $opts.parameters = $QisSession.UserInfo.Sid + "/rep/" + $opts.repid + "/conf/" + $opts.confid;

        if (!$opts.wsfunction) {
            $opts.objid = "/object/" + ($opts.objid ? $opts.objid : $$.Data.ObjId);
            $opts.revid = $opts.revid ? "/revision/" + $opts.revid : "";
            $opts.method = $opts.method ? "/" + $opts.method : "";
            $opts.relation = $opts.relation ? "/" + $opts.relation : "";
            $opts.field = $opts.field ? "?field=" + $opts.field : "";

            $opts.parameters += $opts.objid + $opts.revid + $opts.method + $opts.relation + $opts.field;
        }
        else {
			if ($opts.objid && $opts.wsfunction.toLowerCase().indexOf("csharpmethod") < 0) {
				$opts.parameters += "/Object/" + $opts.objid;
				
				if ($opts.revid) {
					$opts.parameters += "/Revision/" + $opts.revid;
				}
			}

			if ($opts.wsfunction) {
				$opts.parameters += "/" + $opts.wsfunction;
			}
        }
    }
		
	if ( $opts.parameters.indexOf("?") > -1 ) {
		$opts.parameters += "&_t=" + (new Date()).getTime();
	}
	else {
		$opts.parameters += "?_t=" + (new Date()).getTime();
	}

    var wsurl = $QisSession.HTMLBase + "/QisWebService.svc/" + $opts.format + "/" + $opts.type + "/" + $opts.parameters;
	
    $.browser.log(wsurl);
	
    return wsurl;
}

initQisCsharpMethod = function() {
	if (!arguments.length) {
		return null;
	}
	
	var wsmethod = arguments[0];
	var wsqualifier = "CSharpMethod";
	
	if (typeof arguments[1] != "undefined") {
		wsqualifier = arguments[1] ? arguments[1] : "";
	}
	
	wsqualifier = wsqualifier ? wsqualifier + "/" : "";
	
	var wsurl = initQisWebService.call(this, {
		wsfunction: wsqualifier + wsmethod
	});	
		
	return wsurl;
}

handleRequestError = function () {
	if ($QisSession.DEBUGMODE) {
		debugger;
	}
	
	if (!arguments.length) {
		return;
	}
	
	var jqxhr = arguments[0] || {};
	var textStatus = arguments[1] || "";
	var error = arguments[2] || {};
	var wsurl = arguments[3];
	var parms = arguments[4];
	var $obj = arguments[5];

	if (textStatus.match(/abort|parsererror/i)) {
		return;
	}
	
	var $opts = {
		afterLoad: $.noop,
		afterClose: $.noop
	}

	if (arguments[5]) {
		$.extend(true, $opts, arguments[5]);
	}
	
	var $msg = $("<div/>").addClass("request-error");
	var str = "";
	
	switch (jqxhr.status) {
		case 200: str = error.description; break;
		case 500: str = error; break;
		default:  str = jqxhr.responseText; break;
	}
	
	if (!wsurl && !$QisSession.UserInfo.Sid) {
		$msg.append($("<h2/>").html($.LL("Session has expired")))
			.append($("<div/>").html($.LL("Redirecting to login page") + "..."));
		$opts = {
			afterLoad: function(){},
			afterClose: function() {
				if ($msg.find("input:checked").length || offline) {
					endQisSession(location.href);
				}
			}
		}
		$.async(function(){
			endQisSession(location.href);
		}, 2000);
	}
	else if (!jqxhr.status && !jqxhr.readyState && !str) {		
		$QisSession.Offline = true;
		$QisSession.UserInfo = {};
		$.pageCache("clear");
		$.removeCookie("UserRole");

		$msg.append($("<h2/>").html($.LL("Network Connection Lost")))
			.append($("<div/>").html($.LL("Location unreachable:") + " <a href=\"" + location.href + "\">" + location.href + "<a/>"))
			.append($("<div/>").css("margin-top", "10px").loadingText($.LL("Trying to reconnect"), "<h4/>"));
			
		$opts = {
			afterLoad: function(){
				
			}
		}
	}
	else if (str.match(/sorry!/i)) {
		$msg.html(str);
	}
	else {
		var $cnt = $("<div/>").html(str).find(".container");
		
		$msg.append($("<h2/>").html($.LL("Web Service Request Failed")));

		if ($cnt.length) {
		  $msg.append($cnt.find("#detailsViewContainer").html())
			.contents().hide().find("style").remove();
		  delete arguments[3];
		}
		else {
			if (str.match("'")) {
				str = str.slice(str.indexOf("'") + 1, str.lastIndexOf("'"));
			}
			else {
				str = error.description || error;
			}
			
			$.browser.error(str);
			
			$msg.append("<div class=\"message\">" + str + "</div>");

			if (str.match(/not connected/i) || str.match(/expired/i)) {
				$msg.append($("<button/>")
				  .html($.LL("Login"))
				  .css({
					  "float": "left",
					  "clear": "both",
					  "width": "100px"
				  })
				  .button()
				  .on($.browser.click, function(){ endQisSession(); }));
			}
			
			var offline = str.match(/offline/i);
			
			if (str) {
				if (str.match(/unknown object/i)) {
					$msg.find(".message").html($.LL("Unable to find object in current repository configuration.") + "<br/>" 
						+ $.LL("The object is either deleted or belongs to another configuration/repository."));
				}
				var $link = $("<a/>")
					.addClass("stack-trace-link")
					.html($.LL("Details")).appendTo($msg)
					.on($.browser.click, function(){
						$trace.toggle();
						$.async(function(){
							$.fancybox.reposition();
						},100);
					})

				var $trace = $("<div/>")
					.hide()
					.appendTo($msg)
					.addClass("stack-trace")
					.html(str!==$msg.find(".message").html()?str:"")
					.find("#content").css({"margin-left":"0px","font-size":"0.8em"}).end()
					.find("style").remove().end()
					.find(".heading1").remove().end();
					
				if ($obj || wsurl) {
					$trace.append("<br/><div style=\"font-size: 1.25em; font-weight: bold;\" >" + $.LL("Request Info:") + "</div>");
				}
				
				if ($obj) {
					if ($obj.Data) {
						$trace.append("<br/><div style=\"font-size: 9pt;\" ><b style=\"display:inline-block; width: 70px;\">" + $.LL("Name:") + " </b>" + $obj.Data.Name + "</div>");
						$trace.append("<div style=\"font-size: 9pt;\" ><b style=\"display:inline-block; width: 70px;\">" + $.LL("Template:") +" </b>" + $obj.Data.Template + "</div>");
						$trace.append("<div style=\"font-size: 9pt;\" ><b style=\"display:inline-block; width: 70px;\">" + $.LL("Object Id:") + " </b>" + $obj.Data.ObjId + "</div>");
						$trace.append("<div style=\"font-size: 9pt;\" ><b style=\"display:inline-block; width: 70px;\">" + $.LL("Config. Id:") + " </b>" + $obj.Data.ConfId || $QisSession.ConfId + "</div>");
						$trace.append("<div style=\"font-size: 9pt;\" ><b style=\"display:inline-block; width: 70px;\">" + $.LL("Rep. Id:") + " </b>" + $obj.Data.RepId || $QisSession.RepId + "</div>");
					}
				}	
				
				if (wsurl) {
					$trace.append("<br/><div style=\"font-size: 9pt;\" ><b>" + $.LL("Request URL:") + " </b><br/>" + wsurl + "</div>")
					if (parms) {
						$trace.append("<br/><div style=\"font-size: 9pt;\" ><b>" + $.LL("URL Parameters:") + " </b><br/>" + parms + "</div>")
					}
				}
				
				if (!offline) {
					$msg.append("<br/>")
						.append("<input type=\"checkbox\" name=\"endsession\" id=\"endsession\" value=\"endsession\" />")
						.append("<label for=\"endsession\">" + $.LL("End session and re-login") + "</label>")
						.find("input:last").css({
							"position": "relative",
							"top": "3px",
							"left": "-3px"
						});
						
					if (str.match(/id=40/i)) {
						$msg.find("#endsession").attr("checked", "checked").attr("disabled", "disabled");
						$msg.append($("<div/>").css({
									"position": "relative",
									"top": "-21px",
									"left": "215px"
								})
								.append("<input type=\"checkbox\" name=\"changeuser\" id=\"changeuser\" value=\"changeuser\" />")
								.append("<label for=\"changeuser\">" + $.LL("Login with a different user") + "</label>")
								.find("input:last").css({
									"position": "relative",
									"top": "3px",
									"left": "-3px"
								}).end());
					}
					
					$msg.find("input").css("border", 0);
				}
				else {
					$(".jbar").find(".loadingtext")
						.html($.LL("REPOSITORY IS OFFLINE"))
						.css({"position":"static", "text-align":"center"})
				}
			}
		}
		
		$opts = {
			afterLoad: function(){},
			afterClose: function() {
				if ($msg.find("input#endsession:checked").length || offline) {
					if ($msg.find("input#changeuser:checked").length) {
						endQisSession(location.href, false);
					}
					else {
						endQisSession(location.href);
					}
				}
			}
		}
	}
	
	$msg
		.find(".heading1").remove().end()
		.find("style").remove();

	$.overlay(false, true);
	
	$.fancybox($msg, {
		scrolling: "no",
		minWidth: 640,
		minheight: 400,
		afterLoad: $opts.afterLoad,
		afterClose: function(){
			try { $opts.afterClose.call(); }
			catch(e) { $.browser.warn(e)}
			NProgress.done();
		}
	});
}

notifyRequestError = function(){
	if ($QisSession.DEBUGMODE) {
		debugger;
	}
	
	if (!arguments.length) {
		return;
	}
	
	var args = arguments;
	
	var jqxhr = args[0] || {};
	var textStatus = args[1] || "";
	var error = args[2] || {};
	var wsurl = args[3];
	var parms = args[4];
	var $obj = args[5];
	
	if (textStatus.match(/abort|parsererror/i)) {
		return;
	}
	
	$.jnotify({
		title: $.LL("Webservice Request Failed"),
		message: $.LL("Status:") + " " + jqxhr.status + "<br/><br/>" + $.LL("Message:") + " " + error.description || jqxhr.statusText + "<br/><br/><a href=\"#\" class=\"responsetext\">" + $.LL("Details") + "</a>"
	});

	$("a.responsetext")
		.css({
			"color": "white",
			"float": "right",
			"text-decoration": "underline"
		})
		.on($.browser.click, function(e) {
			handleRequestError.apply(this, args);
		})

}
